/**
 * Persistence-log is an easy-use tool which supports customize configuration. 
 * It could save logs with different levels and complexity. This tool could meet 
 * user��s need of log management on software development and running.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.log;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @author jack lee
 * @version 2.0
 * @date 2016-10-01
 */
public class Logger {
	
	private LogProperties prop = null;
	private String dateFormat = "";
	private boolean singleLineLog = true;
	
	private static ThreadLocal<Logger> loggerThread = null;
	private static Logger logger = null;
	private static Map<String,Integer> map = null;
	
	static {
		logger = new Logger();
	}
	private Logger(){
		loggerThread = new ThreadLocal<Logger>();
		prop = LogProperties.getInstance();
		this.dateFormat = prop.getDateFormat();
		this.singleLineLog = prop.isSingleLineLog();
		map = new HashMap<String,Integer>();
		map.put(Logger.DEBUG, 1);
		map.put(Logger.INFO, 2);
		map.put(Logger.WARN, 3);
		map.put(Logger.ERROR, 4);
		map.put(Logger.FATAL, 5);
		map.put(Logger.LOGIN, 6);
		map.put(Logger.VISIT, 7);
		map.put(Logger.DANGEROUS_LOGIN, 8);
		map.put(Logger.SECURITY, 9);
		map.put(Logger.ATTACK_XSS, 10);
		map.put(Logger.ATTACK_SQL_INJECTION, 11);
		map.put(Logger.IMG_HOTLINK, 12);
		map.put(Logger.COOKIE_DIFFERENT_IP, 13);
	}
	
	/**
	 * @return
	 */
	public static Logger getInstance(){
		if ( null == loggerThread.get() ) {
			loggerThread.set(logger);
		}
		return loggerThread.get();
	}
	
	/**
	 * @param head
	 * @param log
	 */
	private synchronized void log(LogData data,Exception e){	
		data.setDate(this.now());
		StringBuilder logString = new StringBuilder();
		if ( this.singleLineLog ) {
			logString.append(data.toSingleLineString());
		} else {
			logString.append(data.toMultipleLineString());
		}
		String result = logString.toString();
		if ( map.get(data.getType()) >= map.get(prop.getMinLevel2File()) ) {
			this.newFolder(prop.getLogFolderPath());
			String logFilePath = this.newLogFile(prop.getLogFolderPath(),data);
			try {
	    		OutputStream fos = new FileOutputStream(logFilePath, true);
	    		OutputStreamWriter osw = new OutputStreamWriter(fos);
	    		PrintWriter writer = new PrintWriter(osw);
	    		writer.write(result);
	    		if ( e != null ) {
	    			e.printStackTrace(writer);
	    		}
	    		writer.close();
	    		osw.close();
	    		fos.close();
	    		writer = null;
	    		osw = null;
	    		fos = null;
			} catch (IOException fe) {
				fe.printStackTrace();
			}
		}
		if ( prop.isDebug() &&
				map.get(data.getType()) >= map.get(prop.getMinLevel2Print()) ) {
			System.out.print(result);
			if ( e != null ) {
				e.printStackTrace();
			}
		}
		
	}
	
	/**
	 * @param log
	 * @return
	 */
	private LogData str2LogData(String log) {
		LogData data = new LogData();
		if ( this.isNull(log) ) {
			log = "Log info is null.";
		}
		data.setDescription(log);
		return data;
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void debug(String log,Exception e){
		this.debug(this.str2LogData(log), e);
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void debug(LogData data,Exception e){
		if ( data != null ) {
			data.setType(Logger.DEBUG);
			this.log(data, e);
		}
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void info(String log,Exception e){
		this.info(this.str2LogData(log), e);
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void info(LogData data,Exception e){
		if ( data != null ) {
			data.setType(Logger.INFO);
			this.log(data, e);
		}
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void warn(String log,Exception e){
		this.warn(this.str2LogData(log), e);
	}

	/**
	 * @param log
	 * @return
	 */
	public void warn(LogData data,Exception e){
		if ( data != null ) {
			data.setType(Logger.WARN);
			this.log(data, e);
		}
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void error(String log,Exception e){
		this.error(this.str2LogData(log), e);
	}

	/**
	 * @param log
	 * @return
	 */
	public void error(LogData data,Exception e){
		if ( data != null ) {
			data.setType(Logger.ERROR);
			this.log(data, e);
		}
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void fatal(String log,Exception e){
		this.fatal(this.str2LogData(log), e);
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void fatal(LogData data,Exception e){
		if ( data != null ) {
			data.setType(Logger.FATAL);
			this.log(data, e);
		}
	}
	
	/**
	 * @param format
	 * @return
	 */
	public Logger setDateFormat(String format){
		if ( !this.isNull(format) ) {
			this.dateFormat = format;
		}
		return this;
	}
	
	/**
	 * @param singleLineLog the singleLineLog to set
	 * @deprecated
	 */
	public Logger setMultipleLineLog() {
		return this;
	}
	
	/**
	 * @param Exception
	 * @return String
	 */
	public String getStackMessage(Exception e) {
		StringBuilder sb = new StringBuilder();
		StackTraceElement[] stackArray = e.getStackTrace();
		for (int i = 0; i < stackArray.length; i++) {
			StackTraceElement element = stackArray[i];
			sb.append(element.toString()).append(LogData.LINE_SEPARATOR);
		}
		return sb.toString();
	}

	/**
	 * @param Throwable
	 * @return String
	 */
	public static String getStackMessage(Throwable e) {
		StringBuilder sb = new StringBuilder();
		StackTraceElement[] stackArray = e.getStackTrace();
		for (int i = 0; i < stackArray.length; i++) {
			StackTraceElement element = stackArray[i];
			sb.append(element.toString()).append(LogData.LINE_SEPARATOR);
		}
		return sb.toString();
	}
	
	/**
	 * Current time.
	 * @return String
	 * */
	public String now(){
		Date now = Calendar.getInstance().getTime();
		return formatDate(now,this.dateFormat);
	}
	
	/**
	 * Current date
	 * @return String
	 * */
	public String date(){
		Date now = Calendar.getInstance().getTime();
		return formatDate(now,"yyyy-MM-dd");
	}
	
	/**
	 * Format date by input format
	 * @param Date date
	 * @param String format like "yyyy-MM-dd"
	 * @return String
	 * */
	public String formatDate(Date date,String format){
		if ( null == date || null == format || "".equals(format) )
			return null;
		SimpleDateFormat formatDate = new SimpleDateFormat(format); 
		return formatDate.format(date);
	}
	
	/**
	 * Create folder by folder path
	 * @param String
	 * */
	public String newFolder(String logfolder){
		if ( this.isNull(logfolder) ) 
			throw new NullPointerException("Log folder path is empty.");
		File f = new File(logfolder);
		if ( f.exists() ) {
			if ( f.isFile() ) {
				f.delete();
			} else if ( f.isDirectory() ) {
				return logfolder;
			}
		} else {
			f.mkdirs();
		}
		return logfolder;
	}
	
	/**
	 * Create log files by folder path
	 * @param String folder
	 * @return String file path
	 * */
	public String newLogFile(String logFolder){
		String path = logFolder + date() + ".log";
		File f = new File(path);
		if ( !f.exists() )
			try {
				f.createNewFile();
			} catch (IOException e) {
				e.printStackTrace();
			}
		f = null;
		return path;
	}
	
	/**
	 * Create log files by folder path
	 * @param String folder
	 * @return String file path
	 * */
	public String newLogFile(String logFolder, LogData data){
		String path = logFolder + data.getType() +"_"+date() + ".log";
		File f = new File(path);
		if ( !f.exists() )
			try {
				f.createNewFile();
			} catch (IOException e) {
				e.printStackTrace();
			}
		f = null;
		return path;
	}
	
	/**
	 * @param s
	 * @return
	 */
	private boolean isNull(String s){
		return s==null || "".equals(s);
	}
	
	/**
	 * info,warn,error,fatal,debug
	 */	
	public static final String INFO = "info";
	public static final String LOGIN = "login";//���¼��¼������IP��ַ����Ϣ
	public static final String VISIT = "visit";//����ʼ�¼������IP��ַ����Ϣ
	
	public static final String WARN = "warn";
	public static final String DANGEROUS_LOGIN = "dangerous-login";//�����¼��¼
	
	public static final String ERROR = "error";
	public static final String FATAL = "fatal";
	public static final String DEBUG = "debug";
	
	public static final String SECURITY = "security";
	public static final String ATTACK_XSS = "xss";//��վ�ű�����
	public static final String ATTACK_SQL_INJECTION = "sql-injection";//SQLע��
	public static final String IMG_HOTLINK = "img-hotlink";//����
	public static final String COOKIE_DIFFERENT_IP = "cookie-different-ip";//����Cookie
	
	/**
	 * @param log
	 * @return
	 */
	public void cookieDifferentIp(String log){
		this.cookieDifferentIp(log, null);
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void cookieDifferentIp(String log,Exception e){
		this.cookieDifferentIp(this.str2LogData(log), e);
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void cookieDifferentIp(LogData data,Exception e){
		if ( data != null ) {
			data.setType(Logger.SECURITY + ":" + Logger.COOKIE_DIFFERENT_IP);
			this.log(data, e);
		}
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void imgHotlink(String log){
		this.imgHotlink(log, null);
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void imgHotlink(String log,Exception e){
		this.imgHotlink(this.str2LogData(log), e);
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void imgHotlink(LogData data,Exception e){
		if ( data != null ) {
			data.setType(Logger.SECURITY + ":" + Logger.IMG_HOTLINK);
			this.log(data, e);
		}
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void sqlInjection(String log){
		this.sqlInjection(log, null);
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void sqlInjection(String log,Exception e){
		this.sqlInjection(this.str2LogData(log), e);
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void sqlInjection(LogData data,Exception e){
		if ( data != null ) {
			data.setType(Logger.SECURITY + ":" + Logger.ATTACK_SQL_INJECTION);
			this.log(data, e);
		}
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void xss(String log){
		this.xss(log, null);
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void xss(String log,Exception e){
		this.xss(this.str2LogData(log), e);
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void xss(LogData data,Exception e){
		if ( data != null ) {
			data.setType(Logger.SECURITY + ":" + Logger.ATTACK_XSS);
			this.log(data, e);
		}
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void security(String log){
		this.security(log, null);
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void security(String log,Exception e){
		this.security(this.str2LogData(log), e);
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void security(LogData data,Exception e){
		if ( data != null ) {
			data.setType(Logger.SECURITY);
			this.log(data, e);
		}
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void dangerousLogin(String log){
		this.dangerousLogin(log, null);
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void dangerousLogin(String log,Exception e){
		this.dangerousLogin(this.str2LogData(log), e);
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void dangerousLogin(LogData data,Exception e){
		if ( data != null ) {
			data.setType(Logger.WARN + ":" + Logger.DANGEROUS_LOGIN);
			this.log(data, e);
		}
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void visit(String log){
		this.visit(log, null);
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void visit(String log,Exception e){
		this.visit(this.str2LogData(log), e);
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void visit(LogData data,Exception e){
		if ( data != null ) {
			data.setType(Logger.INFO + ":" + Logger.VISIT);
			this.log(data, e);
		}
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void login(String log){
		this.login(log, null);
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void login(String log,Exception e){
		this.login(this.str2LogData(log), e);
	}
	
	/**
	 * @param log
	 * @return
	 */
	public void login(LogData data,Exception e){
		if ( data != null ) {
			data.setType(Logger.INFO + ":" + Logger.LOGIN);
			this.log(data, e);
		}
	}
	
	
}
