/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more 
 * solutions for quick-development of complex application. 
 * 
 * Persistence2-config is a basic toolkit of Persistence2 framework. It 
 * provides management of settings on ORM and SQL-MAPPING.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the “Software”), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.config;

/**
 * @author jack lee
 * @version 1.0
 * @date 2016-09-30
 */
public class CommonTools {

	/**
	 * Determine if the string length is correct
	 * @param str
	 * @param minLength 
	 * @param maxLength
	 * @return
	 */
	public final static boolean checkLength(String str, int minLength, int maxLength){
		if ( isNull(str) ) {
			return false;
		}
		int length = str.length();
		if ( (minLength > 0 && length < minLength) 
				|| (maxLength > 1 && length > maxLength) ) {
			return false;
		}
		return true;
	}
	
	/**
	 * Determine if the string is double type and the value is correct
	 * @param str
	 * @param minValue
	 * @param maxValue
	 * @return
	 */
	public final static boolean checkDoubleValue(String str, Double minValue, Double maxValue){
		if ( isNull(str) ) {
			return false;
		}
		if ( !isNumber(str) ) {
			return false;
		}
		double value = Double.valueOf(str);
		if ( maxValue != null ) {
			double r = maxValue-value;
			if ( !((r>0) || Math.abs(r) < 0.01) )  {
				return false;
			} 
		} 
		if ( minValue != null ){
			double r = value-minValue;
			if ( !((r>0) || Math.abs(r) < 0.01) )  {
				return false;
			} 
		}
		return true;
	}
	
	/**
	 * Determine if the string is float type and the value is correct
	 * @param str
	 * @param minValue
	 * @param maxValue
	 * @return
	 */
	public final static boolean checkFloatValue(String str, Float minValue, Float maxValue){
		if ( isNull(str) ) {
			return false;
		}
		if ( !isNumber(str) ) {
			return false;
		}
		float value = Float.valueOf(str);
		if ( maxValue != null ) {
			float r = maxValue-value;
			if ( !((r>0) || Math.abs(r) < 0.01) )  {
				return false;
			} 
		} 
		if ( minValue != null ){
			float r = value-minValue;
			if ( !((r>0) || Math.abs(r) < 0.01) )  {
				return false;
			} 
		}
		return true;
	}
	
	/**
	 * Determine if the string is long type and the value is correct
	 * @param str
	 * @param minValue
	 * @param maxValue
	 * @return
	 */
	public final static boolean checkLongValue(String str, Long minValue, Long maxValue){
		if ( isNull(str) ) {
			return false;
		}
		if ( !isNumber(str) ) {
			return false;
		}
		long value = Long.valueOf(str);
		if ( maxValue != null ) {
			if ( value > maxValue ) {
				return false;
			}
		} 
		if ( minValue != null ){
			if ( value < minValue ) {
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Determine if the string is int type and the value is correct
	 * @param str
	 * @param minValue
	 * @param maxValue
	 * @return
	 */
	public final static boolean checkIntegerValue(String str, Integer minValue, Integer maxValue){
		if ( isNull(str) ) {
			return false;
		}
		if ( !isNumber(str) ) {
			return false;
		}
		int value = Integer.valueOf(str);
		if ( maxValue != null ) {
			if ( value > maxValue ) {
				return false;
			}
		} 
		if ( minValue != null ){
			if ( value < minValue ) {
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Determine if the string is short type and the value is correct
	 * @param str
	 * @param minValue
	 * @param maxValue
	 * @return
	 */
	public final static boolean checkShortValue(String str, Short minValue, Short maxValue){
		if ( isNull(str) ) {
			return false;
		}
		if ( !isNumber(str) ) {
			return false;
		}
		short value = Short.valueOf(str);
		if ( maxValue != null ) {
			if ( value > maxValue ) {
				return false;
			}
		} 
		if ( minValue != null ){
			if ( value < minValue ) {
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Determines whether the string match the regular expression
	 * @param value string
	 * @param valueRule regular expression
	 * @return
	 */
	public static final boolean matchValueRule(String value, String valueRule){
		if ( isNull(value) || isNull(valueRule) ) {
			return false;
		}
		return value.matches(valueRule);
	}
	
	/**
	 * Determines whether the string is null
	 * @param Object
	 * @return
	 */
	public final static boolean isNull(Object obj){
		if ( obj == null ) {
			return true;
		}
		if ( obj.getClass() == String.class && "".equals(obj.toString()) ) {
			return true;
		}
		return false;
	}
	
	/**
	 * Determines whether the string is number type
	 * @param Object
	 * @return
	 */
	public final static boolean isNumber(String obj){
		if ( obj == null ) {
			return false;
		}
		if ( obj.matches("\\d+") ) {
			return true;
		}
		return false;
	}
	
	/**
	 * Determines whether the string is Integer type
	 * @param Object
	 * @return
	 */
	public final static boolean isIntegerNumber(Object obj){
		if ( obj == null ) {
			return false;
		}
		if ( obj.getClass() == Integer.class ||
				obj.getClass() == int.class ) {
			return true;
		}
		return false;
	}
		
	public static void main(String[] args) {
		System.out.println(isNumber("200"));
		System.out.println(isNumber("200o"));
		Integer value = new Integer(500);
		System.out.println(Double.valueOf(value.toString()));
		
		int x = 10;
		System.out.println((Integer)x);
	}
	
	
}
