/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more 
 * solutions for quick-development of complex application. 
 * 
 * Persistence2-config is a basic toolkit of Persistence2 framework. It 
 * provides management of settings on ORM and SQL-MAPPING.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the “Software”), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.config;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.config.bean.Action;
import com.nonesole.persistence2.config.bean.Actions;
import com.nonesole.persistence2.config.bean.OSMObject;

/**
 * This is the entrance of persistence2-config toolkit.It is not a self-starting
 * framework. It could work when the outside functions call it.
 * @author jack
 * @version 1.0
 */
public class ConfigParser {

	private Logger logger = null;
	private String filePath = null;
	private InputStream inputStream = null;
	private boolean isStream = false;
	private Document doc = null;
	
	/**
	 * Configuration parser
	 * @param filePath
	 * @throws ParserException
	 * @throws ParserConfigurationException
	 * @throws SAXException
	 * @throws IOException
	 */
	public ConfigParser(String filePath) 
			throws ParserException, ParserConfigurationException, SAXException, IOException{
		if ( logger == null ) {
			logger = Logger.getInstance();
		}
		StringBuilder error = new StringBuilder();
		error.append("The loadFile method of ConfigParser class occurs");
		this.filePath = filePath;
		logger.debug("ConfigParser:filePath"+this.filePath,null);
		doc = XMLHandler.getInstance().loadFile(this.filePath);
		isStream = false;
	}
	
	/**
	 * Configuration parser
	 * @param inputStream
	 * @throws ParserException
	 * @throws ParserConfigurationException
	 * @throws SAXException
	 * @throws IOException
	 */
	public ConfigParser(InputStream inputStream) 
			throws ParserException, ParserConfigurationException, SAXException, IOException{
		if ( logger == null ) {
			logger = Logger.getInstance();
		}
		StringBuilder error = new StringBuilder();
		error.append("The loadFile method of ConfigParser class occurs");
		this.inputStream = inputStream;
		doc = XMLHandler.getInstance().loadFile(this.inputStream);
		isStream = true;
	}
		
	/**
	 * Count the number of child nodes
	 * @param parentNode
	 * @param childNodeName
	 * @throws ParserException 
	 */
	private int countChildNodeNumber(Node parentNode,String childNodeName) 
			throws ParserException{
		List<Node> nodes = XMLHandler.getInstance().getChildNodes(parentNode, childNodeName);
		if ( null == nodes || nodes.isEmpty() ) {
			return 0; 
		} else {
			if ( nodes.size() != 1 ) {
				StringBuilder error = new StringBuilder();
				error.append("The ").append(childNodeName)
				     .append(" nodes of ").append(parentNode.getNodeName())
				     .append(" is more than 1.");
				throw new ParserException(error.toString());
			}
		}
		return nodes.size();
	}
	
	/**
	 * Parse node and build list of object-sql-mapping object 
	 * @param actionNodes
	 * @param actions
	 * @throws ParserException 
	 */
	public List<OSMObject> osmParser() throws ParserException{
		logger.debug("ConfigParser:osmParser",null);
		int osmNodeNumber = this.countChildNodeNumber(doc, OBJECT_SQL_MAPPING);
		if ( osmNodeNumber < 1 ) {
			return null;
		}
		List<Node> osmObjectNodes = XMLHandler.getInstance()
										.getChildNodes(doc, OBJECT_SQL_MAPPING, OBJECT);
		if ( osmObjectNodes != null && !osmObjectNodes.isEmpty() ) {
			List<OSMObject> osml = new ArrayList<OSMObject>();
			for ( Node n : osmObjectNodes ) {
				osml.add(OSMParser.getInstance().parse(n));
			}
			return osml;
		}
		return null;
	}
			
	/**
	 * Parse node and build Actions object 
	 * @param actionNodes
	 * @param actions
	 * @throws ParserException 
	 */
	public Actions actionsParser() throws ParserException {
		logger.debug("ConfigParser:actionsParser",null);
		Actions actions = null;
		int actNodeNumber = this.countChildNodeNumber(doc, ACTIONS);
		if ( actNodeNumber < 1 ) {
			return actions;
		}
		
		List<Node> actionNodes = XMLHandler.getInstance().getChildNodes(doc, ACTIONS, ACTION);
		if ( actionNodes != null && !actionNodes.isEmpty() ) {
			actions = new Actions();
			//List<Action> actionList = new ArrayList<Action>();
			Action action = null;
			for ( Node n : actionNodes ) {
				action = ActionParser.getInstance().parse(n);
				//actionList.add(action);
				actions.put(action.getName(), action);
			}
			
		}
		return actions;
	}

	/**
	 * Reloading configuration
	 * @throws ParserException 
	 * @throws IOException 
	 * @throws SAXException 
	 * @throws ParserConfigurationException 
	 */
	public void reload() 
			throws ParserException, ParserConfigurationException, SAXException, IOException{
		StringBuilder error = new StringBuilder();
		error.append("The reload method of ConfigParser class occurs");
		if ( !isStream ) {
			doc = XMLHandler.getInstance().loadFile(this.filePath);
		} else {
			doc = XMLHandler.getInstance().loadFile(this.inputStream);
		}
		this.load();
	}
	
	/**
	 * Loading configuration
	 * @throws ParserException 
	 */
	public void load() throws ParserException{
		this.osmParser();//����OSM����
		this.actionsParser();//�����Զ���DAO����
	}
	
	/**
	 * @param filePath
	 */
	public void setPath(String filePath){
		this.filePath = filePath;
	}
	
	/**
	 * @param inputStream
	 */
	public void setPath(InputStream inputStream){
		this.inputStream = inputStream;
	}
	

	/**
	 * Signs of configuration in XML files
	 */
	public static final String ACTIONS = "actions";
	public static final String ACTION = "action";
	public static final String ACTION_NAME = "name";
	public static final String ACTION_TRANSACTION = "transaction";
	public static final String ACTION_ORM = "orm-action";
	public static final String ACTION_INPUT = "input";
	public static final String ACTION_INPUT_OBJECT = "object";
	public static final String ACTION_INPUT_OBJECT_PROPERTY = "property";
	public static final String ACTION_INPUT_OBJECT_ACTION_NAME = "action";
	public static final String ACTION_INPUT_OBJECT_MAPPING = "mapping";
	public static final String ACTION_INPUT_OBJECT_MAPPING_SOURCE = "source";
	public static final String ACTION_INPUT_OBJECT_MAPPING_TARGET = "target";
	public static final String ACTION_INPUT_OBJECT_MAPPING_NAME = "name";
	public static final String ACTION_INPUT_OBJECT_MAPPING_CLASS = "class";
	public static final String ACTION_INPUT_OBJECT_MAPPING_NOT_NULL = "not-null";
	public static final String ACTION_INPUT_OBJECT_MAPPING_DEFAULT_VALUE = "default-value";
	public static final String ACTION_INPUT_OBJECT_MAPPING_MIN_VALUE = "min-value";
	public static final String ACTION_INPUT_OBJECT_MAPPING_MAX_VALUE = "max-value";
	public static final String ACTION_INPUT_OBJECT_MAPPING_MIN_LENGTH = "min-length";
	public static final String ACTION_INPUT_OBJECT_MAPPING_MAX_LENGTH = "max-length";
	public static final String ACTION_INPUT_OBJECT_MAPPING_WHERE = "where";
	public static final String ACTION_INPUT_OBJECT_MAPPING_VALUE_RULE = "value-rule";
	public static final String ACTION_INPUT_OBJECT_MAPPING_TARGET_NAME = "name";
	
	public static final String ACTION_INPUT_OBJECT_MAPPING_SQL_PART = "sql-part";
	
	public static final String ACTION_INPUT_OBJECT_NAME = "name";
	public static final String ACTION_INPUT_OBJECT_CLASS = "class";
	public static final String ACTION_OUTPUT = "output";
	public static final String ACTION_OUTPUT_OBJECT = "object";
	public static final String ACTION_OUTPUT_OBJECT_PROPERTY = "property";
	public static final String ACTION_OUTPUT_OBJECT_CLASS = "object-class";
	public static final String ACTION_OUTPUT_OBJECT_ITEM_CLASS = "item-class";
	public static final String ACTION_OUTPUT_OBJECT_MAPPING = "mapping";
	public static final String ACTION_OUTPUT_OBJECT_MAPPING_SOURCE = "source";
	public static final String ACTION_OUTPUT_OBJECT_MAPPING_SOURCE_NAME = "name";
	public static final String ACTION_OUTPUT_OBJECT_MAPPING_TARGET = "target";
	public static final String ACTION_OUTPUT_OBJECT_MAPPING_TARGET_NAME = "name";
	public static final String ACTION_OUTPUT_OBJECT_MAPPING_TARGET_CLASS = "class";

	public static final String ACTION_SQL = "sql";
	
	public static final String OBJECT_SQL_MAPPING = "object-sql-mapping";
	public static final String OBJECT = "object";
	public static final String OBJECT_CLASS = "class";
	public static final String OBJECT_TABLE = "table";
	public static final String OBJECT_PRIMERY_KEY = "primery-key";
	
}
