/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more 
 * solutions for quick-development of complex application. 
 * 
 * Persistence2-datasource is a connection-pool toolkit of Persistence2 framework. 
 * It provides service of allocating, managing and releasing database connections.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.connection.pool;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.LinkedList;
import java.util.List;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.connection.exception.OperationsException;
import com.nonesole.persistence2.connection.pool.parser.ConnectionPoolProperties;
import com.nonesole.persistence2.connection.pool.parser.DriverProperties;

/**
 * There is a one-to-one relationship between The connection pool object and 
 * the data source object. All connections in the pool are the same datasource.
 * We use ThreadLocal here to provide thread for each connection. 
 * @author Jack Lee
 * @version 1.0 - build on 2016-05-07
 */
public class DefaultConnectionPool {

	//JDBC configuration
	private DriverProperties connConfig = null;
	//Pool configuration
	private ConnectionPoolProperties connPoolConfig = null;
	//Free connections
	private List<Connection> freeConnections = null;
	//Active connections
	private List<Connection> activeConnections = null;
	//ThreadLocal of connection
	private static ThreadLocal<Connection> THREAD_LOCAL = new ThreadLocal<Connection>();
	//Datasource of pool
	private PoolDataSource datasource = null;
	//When there is no active connection, system will check every 'waitTime' millisecond
	private int waitTime;
	
	/**
	 * Constructor
	 * @param connConfig
	 * @param connPoolConfig
	 * @throws OperationsException
	 * @throws ClassNotFoundException 
	 * @throws IllegalAccessException 
	 * @throws InstantiationException 
	 * @throws SQLException
	 */
	public DefaultConnectionPool(
			DriverProperties connConfig,ConnectionPoolProperties connPoolConfig) 
					throws OperationsException, InstantiationException, IllegalAccessException, ClassNotFoundException{
		this.connPoolConfig = connPoolConfig;
		this.connConfig = connConfig;
		this.freeConnections = new LinkedList<Connection>();
		this.activeConnections = new LinkedList<Connection>();
		datasource = new PoolDataSource(this);
		waitTime = 1000;
		this.initPool();
	}
	
	/**
	 * Initialize the configuration of connection pool.
	 * @throws OperationsException
	 * @throws SQLException 
	 */
	public void initPool() throws OperationsException{
		if ( !connConfig.isValid() ) {
			throwExcpetion("DataSource Config is failure.",null);
		}
		if ( !connPoolConfig.isValid() ) {
			throwExcpetion("Connection Pool Config is failure.",null);
		}
		int size = connPoolConfig.getInitialPoolSize();
		Connection conn = null;
		for ( int i = 0 ; i < size ; i++ ){
			try {
				conn = datasource.getConnection();
			} catch (SQLException e) {
				throw new OperationsException("Getting connection is failure.",e.fillInStackTrace());
			}
			freeConnections.add(conn);
		}
	}
	
	/**
	 * Private connection from pool
	 * @return
	 * @throws SQLException
	 * @throws InterruptedException
	 */
	public synchronized Connection getConnection() throws OperationsException{
		Connection conn = null;
		if ( freeConnections != null && !freeConnections.isEmpty() ) {
			conn = freeConnections.get(0);
		}
		if ( null != conn ) {
			activeConnections.add(conn);
			freeConnections.remove(conn);
			THREAD_LOCAL.set(conn);
		} else {
			int connSize = freeConnections.size() + activeConnections.size();
			if ( connSize < connPoolConfig.getMaxPoolSize() ) {
				try {
					conn = datasource.getConnection();
				} catch (SQLException e) {
					throw new OperationsException("Getting connection is failure.",e.fillInStackTrace());
				}
				freeConnections.add(conn);
				THREAD_LOCAL.set(conn);
			} else {
				Logger.getInstance().warn("Warning! The connection pool is full!", null);
				try {
					wait(waitTime);
				} catch (InterruptedException e) {
					throw new OperationsException("Waiting of get connection is failure.",e.fillInStackTrace());
				}
			}
		}
		return conn;
	}
	
	/**
	 * Release connection
	 * @param conn
	 */
	public synchronized void release(Connection conn){
		if ( null != conn && activeConnections.indexOf(conn) > -1 ) {
			activeConnections.remove(conn);
			freeConnections.add(conn);
			THREAD_LOCAL.remove();
            notifyAll();
		} 
	}
	
	/**
	 * Throw OperationException
	 * @param message String 
	 * @param cause Throwable
	 * */
	private void throwExcpetion(String message,Throwable cause) 
			throws OperationsException{
		throw new OperationsException(message,cause);
	}

	/**
	 * @return the connConfig
	 */
	public DriverProperties getConnConfig() {
		return connConfig;
	}

	/**
	 * @param connConfig the connConfig to set
	 */
	public void setConnConfig(DriverProperties connConfig) {
		this.connConfig = connConfig;
	}

	/**
	 * @return the connPoolConfig
	 */
	public ConnectionPoolProperties getConnPoolConfig() {
		return connPoolConfig;
	}

	/**
	 * @param connPoolConfig the connPoolConfig to set
	 */
	public void setConnPoolConfig(ConnectionPoolProperties connPoolConfig) {
		this.connPoolConfig = connPoolConfig;
	}
	
	/**
	 * @return
	 */
	public String getDataSourceName(){
		return connConfig.getConnectionPoolName();
	}
}
