/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more 
 * solutions for quick-development of complex application. 
 * 
 * Persistence2-datasource is a connection-pool toolkit of Persistence2 framework. 
 * It provides service of allocating, managing and releasing database connections.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.connection.pool;

import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.SQLFeatureNotSupportedException;
import java.util.logging.Logger;

import javax.sql.DataSource;

import com.nonesole.persistence2.connection.exception.OperationsException;
import com.nonesole.persistence2.connection.pool.parser.DriverProperties;

/**
 * Local data source
 * <pre>
 * 1.Create new datasource from jdbc config
 * &#9;	DriverDataSource dds = new DriverDataSource(new DriverProperties());
 * 2.Create new datasource from context of server just like Tomcat��
 * &#9;	Context ctx = new InitialContext();
 * &#9;	DataSource ds = ctx.lookup("jdbc/db");
 * 3.Create new datasource from Persistence frame
 * &#9;	ConnectionPoolServer.initPool();
 * &#9;	DataSource ds = ConnectionPoolContext.getDataSource("mysql");
 * </pre>
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-04
 */
public class DefaultDataSource implements DataSource {

	//jdbc config object
	private DriverProperties driver;
	//Connection object
	private Connection conn;
	//Exception head
	private final static String REPORTER = "DriverDataSource ";
	private final static String KEY_ENTER = "\r\n";
		
	/**
	 * Construction 
	 * @param driver jdbc config object 
	 * @throws OperationsException 
	 * */
	public DefaultDataSource(DriverProperties driver) throws OperationsException{
		this.driver = driver;
		if( !this.driver.isValid() )
			throw new OperationsException("DataSource Config is failure.");
	}
	
	/**
	 * Initialize Connection
	 * @throws OperationsException 
	 * */
	private synchronized final void initConnection() throws OperationsException {
		try {
			Class.forName(this.driver.getDriverClass()).newInstance();
			conn = DriverManager.getConnection(
					driver.getUrl(),driver.getUser(),driver.getPwd());
		} catch (InstantiationException e) {
			throwExcpetion(REPORTER + e.getMessage() + KEY_ENTER,
					e.fillInStackTrace());
		} catch (IllegalAccessException e) {
			throwExcpetion(REPORTER + e.getMessage() + KEY_ENTER,
					e.fillInStackTrace());
		} catch (ClassNotFoundException e) {
			throwExcpetion(REPORTER + e.getMessage() + KEY_ENTER,
					e.fillInStackTrace());
		} catch (SQLException e) {
			throwExcpetion(REPORTER + e.getMessage() + KEY_ENTER,
					e.fillInStackTrace());
		}
	}
	
	/* (non-Javadoc)
	 * @see javax.sql.DataSource#getConnection()
	 */
	public synchronized Connection getConnection(){
		return getConnection(null,null);
	}

	/* (non-Javadoc)
	 * @see javax.sql.DataSource#getConnection(java.lang.String, java.lang.String)
	 */
	public synchronized Connection getConnection(String username, String password) {
		if ( null != username )
			driver.setUser(username);
		if ( null != password )
			driver.setPwd(password);
		try {
			initConnection();
		} catch (OperationsException e) {
			com.nonesole.log.Logger.getInstance().error("There is an error on creating connection.", e);
		}
		return conn;
	}
	
	/**
	 * @param message String 
	 * @param cause Throwable
	 * */
	private void throwExcpetion(String message,Throwable cause) throws OperationsException{
		throw new OperationsException(message,cause);
	}
	
	private PrintWriter writer;
	
	/* (non-Javadoc)
	 * @see javax.sql.DataSource#getLogWriter()
	 */
	public PrintWriter getLogWriter() throws SQLException {
		return writer;
	}
	
	/* (non-Javadoc)
	 * @see javax.sql.DataSource#setLogWriter(java.io.PrintWriter)
	 */
	public void setLogWriter(PrintWriter out) throws SQLException {
		writer = out;
	}

	private int maxLoginTime = 0; 

	/* (non-Javadoc)
	 * @see javax.sql.DataSource#getLoginTimeout()
	 */
	public int getLoginTimeout() throws SQLException {
		return maxLoginTime;
	}
	
	/* (non-Javadoc)
	 * @see javax.sql.DataSource#setLoginTimeout(int)
	 */
	public void setLoginTimeout(int seconds) throws SQLException {
		maxLoginTime = seconds;
	}

	/**
	 * Get driver configuration
	 * @see com.nonesole.persistence.connectionpool.IDriverDataSource#getDriverProperties()
	 * */
	public DriverProperties getDriverProperties() {
		return driver;
	}

	@Override
	public Logger getParentLogger() throws SQLFeatureNotSupportedException {
		return null;
	}

	@Override
	public boolean isWrapperFor(Class<?> iface) throws SQLException {
		return conn.isWrapperFor(iface);
	}

	@Override
	public <T> T unwrap(Class<T> iface) throws SQLException {
		return conn.unwrap(iface);
	}

}
