/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more 
 * solutions for quick-development of complex application. 
 * 
 * Persistence2-datasource is a connection-pool toolkit of Persistence2 framework. 
 * It provides service of allocating, managing and releasing database connections.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.connection.pool;

import java.sql.Connection;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.connection.exception.OperationsException;
import com.nonesole.persistence2.connection.pool.parser.ConnectionPoolConfigParser;
import com.nonesole.persistence2.connection.pool.parser.DataSourceConfigParser;
import com.nonesole.persistence2.connection.pool.parser.DriverProperties;

/**
 * MultiConnectionPool is a container of DefaultConnectionPool.Users could manage connections 
 * which is from different datasource by MultiConnectionPool.
 * @author Jack Lee
 * @version 1.0 - build on 2016-05-07
 */
public class MultiConnectionPool {

	//JDBC configuration
	private List<DriverProperties> jdbcConfigs = null;
	//Parser of connection pool
	private ConnectionPoolConfigParser connpoolParser = null;
	//HashMap of DefaultConnectionPool
	private Map<String,DefaultConnectionPool> poolMap = null;
	//The name of default datasource
	private String defaultDataSourceName = null;
	
	/**
	 * Path of configuration
	 */
	private static final String DATA_SOURCE_RELATIVE_PATH = "connection.xml"; 
	
	private static MultiConnectionPool pool = null;
	static {
		pool = new MultiConnectionPool();
	}
	
	/**
	 * Constructor
	 */
	private MultiConnectionPool(){
		try {
			this.init();
		} catch (Exception e) {
			Logger.getInstance().error("There is an error on creating MultiConnectionPool.", e);
			//e.printStackTrace();
		}
	}
	
	/**
	 * Get singleton object
	 * @return
	 */
	public static MultiConnectionPool getInstance(){
		return pool;
	}
	
	/**
	 * Initialize the connection pool by parsing configuration and building pools of different datasource.
	 * @throws OperationsException 
	 * @throws ClassNotFoundException 
	 * @throws IllegalAccessException 
	 * @throws InstantiationException 
	 */
	private void init() throws OperationsException, InstantiationException, IllegalAccessException, ClassNotFoundException{
		DataSourceConfigParser dscp = DataSourceConfigParser.getInstance();
		dscp.setRelativePath(DATA_SOURCE_RELATIVE_PATH);
		try {
			dscp.parse();
		} catch (Exception e) {
			throw new OperationsException(e);
		}
		jdbcConfigs = dscp.getDataSources();
		jdbcConfigs = this.getEffectiveDrivers(jdbcConfigs);
		
		connpoolParser = ConnectionPoolConfigParser.getInstance()
				.setRelativePath(DATA_SOURCE_RELATIVE_PATH);
		try {
			connpoolParser.parse();
		} catch (Exception e) {
			throw new OperationsException(e);
		}
		poolMap = new HashMap<String,DefaultConnectionPool>();
		DefaultConnectionPool connpool = null;
		DriverProperties dp = null;
		for ( int i = 0 ; i < jdbcConfigs.size() ; i++ ) {
			dp = jdbcConfigs.get(i);
			connpool = new DefaultConnectionPool(dp,
					connpoolParser.getConnectionPool(
							dp.getConnectionPoolName()));
			poolMap.put(connpool.getDataSourceName(), connpool);
			if ( i == 0 ) {
				//�����е�һ����ȡ�����ԴΪĬ�����Դ 
				this.defaultDataSourceName = connpool.getDataSourceName();
			}
		}
	}
	
	/**
	 * Pick up effective JDBC configuration drivers for DataSource.
	 * @param l list of JDBC configuration drivers from configuration files
	 * @return effective JDBC configuration drivers
	 * @throws OperationsException 
	 * */
	private final List<DriverProperties> getEffectiveDrivers(List<DriverProperties> l) 
			throws OperationsException{
		List<DriverProperties> d = new ArrayList<DriverProperties>();
		for ( DriverProperties dp : l ) {
			if ( dp.isAssociateConnectionPool() ) {
				if ( null != dp.getConnectionPoolName() 
						&& !"".equals(dp.getConnectionPoolName()) ) {
					d.add(dp);
				}
			}
		}
		l = null;
		return d;
	}
	
	/**
	 * Get connection of a datasource by datasource name
	 * @param dataSourceName
	 * @return
	 * @throws OperationsException 
	 */
	public synchronized Connection getConnection(String dataSourceName) 
			throws OperationsException{
		DefaultConnectionPool connpool = null;
		if ( null != poolMap ) {
			connpool = poolMap.get(dataSourceName);
			Connection conn = connpool.getConnection();
			if ( conn == null ) {
				return this.getConnection(dataSourceName);
			}
			return conn;
		}
		return null;
	}
	
	/**
	 * Get name of default datasource
	 * @return ���Դ���
	 */
	public String getDefaultDataSourceName(){
		return this.defaultDataSourceName;
	}
}
