/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more 
 * solutions for quick-development of complex application. 
 * 
 * Persistence2-datasource is a connection-pool toolkit of Persistence2 framework. 
 * It provides service of allocating, managing and releasing database connections.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.connection.pool.parser;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import com.nonesole.persistence2.config.XMLHandler;
import com.nonesole.persistence2.connection.exception.OperationsException;
import com.nonesole.persistence2.connection.pool.Constant;
import com.nonesole.persistence2.connection.tools.FileToolkit;
import com.nonesole.persistence2.connection.tools.ReflectToolkit;

/**
 * Parse Configuration of DataSource from xml files
 * @author JACK LEE
 * @version 1.1 - build on 2016-05-02
 */
public class DataSourceConfigParser {

	private static DataSourceConfigParser dscp;
	
	private String absolutePath;
	private boolean isAbsolutePath = false;
	
	private String relativePath;
	private boolean isRelativePath = false;
	
	private InputStream xmlInputStream;
	private boolean isInputStream = false;

	private static List<DriverProperties> DATA_SOURCE_LIST;
	
	static {
		dscp = new DataSourceConfigParser();
		DATA_SOURCE_LIST = new ArrayList<DriverProperties>();
	}
	
	/**
	 * Constructor
	 */
	private DataSourceConfigParser(){}
	
	/**
	 * Get singleton object
	 * @return
	 */
	public static DataSourceConfigParser getInstance(){
		return dscp;
	}
	
	/**
	 * Set absolute path
	 * @param absolutePath
	 * @return DataSourceConfigParser
	 */
	public DataSourceConfigParser setAbsolutePath(String absolutePath){
		this.absolutePath = absolutePath;
		this.isAbsolutePath = true;
		this.isInputStream = false;
		this.isRelativePath = false;
		return dscp;
	}
	
	/**
	 * Set relative path
	 * @param absolutePath
	 * @return DataSourceConfigParser
	 */
	public DataSourceConfigParser setRelativePath(String relativePath){
		this.relativePath = relativePath;
		this.isAbsolutePath = false;
		this.isInputStream = false;
		this.isRelativePath = true;
		return dscp;
	}
	
	/**
	 * Set input stream
	 * @param inputStream
	 * @return DataSourceConfigParser
	 */
	public DataSourceConfigParser setInputStream(InputStream inputStream){
		this.xmlInputStream = inputStream;
		this.isAbsolutePath = false;
		this.isInputStream = true;
		this.isRelativePath = false;
		return dscp;
	}
	
	
	/**
	 * Parse xml files
	 * */
	public final void parse() throws OperationsException{
		Document doc = null;
		try {
			if ( this.isAbsolutePath ) {
				doc = XMLHandler.getInstance().loadFile(this.absolutePath);
			} else if ( this.isRelativePath ) {
				InputStream is = FileToolkit.getResourceAsStream(this.relativePath);
				doc = XMLHandler.getInstance().loadFile(is);
			} else if ( this.isInputStream ) {
				doc = XMLHandler.getInstance().loadFile(this.xmlInputStream);
			} else {
				throw new OperationsException("XML file of config is null.");
			}
		} catch ( Exception e ) {
			throw new OperationsException("File parsing is failed.",e.fillInStackTrace());
		}
		
		Node root = doc.getDocumentElement();
		List<Node> l = XMLHandler.getInstance().getNodes(root, 
				Constant.DATASOURCE_CONTAINER_NAME);
		
		if ( null == l )
			throw new OperationsException("Config file has no config information.");
		
		DriverProperties dp = null;
		for ( Node n : l ) {
			dp = parseParameter(n);
			dp.setDataSourceName(n.getAttributes()
					.getNamedItem(Constant.DATASOURCE_NAME)
					.getNodeValue());
			DATA_SOURCE_LIST.add(dp);
		}
	}
	
	/**
	 * Get DriverProperties object from parsing xml files
	 * @param n Node
	 * @return DriverProperties
	 * */
	private final DriverProperties parseParameter(Node n) throws OperationsException{
		Map<String,Object> m = new HashMap<String,Object>();
		try {
			ConnectionPoolParserUtil.parseParameter(n,m,Constant.PARAMETER);
		} catch (Exception e) {
			throw new OperationsException("Parameter parsing is failed.",e.fillInStackTrace());
		}
		
		try {
			return (DriverProperties) ReflectToolkit
						.parseMapToObject(m,DriverProperties.class);
		} catch (Exception e) {
			throw new OperationsException("Map Transfering is failed.",e.fillInStackTrace());
		}
	}
	
	/**
	 * Get configuration list of DataSource
	 * @return list of DriverProperties
	 * */
	public final List<DriverProperties> getDataSources(){
		return DATA_SOURCE_LIST;
	}
	
}
