/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql;

import java.sql.Connection;

import com.nonesole.persistence2.connection.exception.OperationsException;
import com.nonesole.persistence2.connection.pool.MultiConnectionPool;
import com.nonesole.persistence2.sql.custom.DefaultArrayValueAction;
import com.nonesole.persistence2.sql.custom.DefaultDyadicArrayValueAction;
import com.nonesole.persistence2.sql.custom.DefaultDyadicListValueAction;
import com.nonesole.persistence2.sql.custom.DefaultListValueAction;
import com.nonesole.persistence2.sql.custom.DefaultMapAction;
import com.nonesole.persistence2.sql.custom.DefaultPojoAction;
import com.nonesole.persistence2.sql.custom.DefaultSingleValueAction;
import com.nonesole.persistence2.sql.custom.DefaultVoidAction;
import com.nonesole.persistence2.sql.define.IArrayValueAction;
import com.nonesole.persistence2.sql.define.IDyadicArrayValueAction;
import com.nonesole.persistence2.sql.define.IDyadicListValueAction;
import com.nonesole.persistence2.sql.define.IListValueAction;
import com.nonesole.persistence2.sql.define.IMapAction;
import com.nonesole.persistence2.sql.define.IOrmAction;
import com.nonesole.persistence2.sql.define.IOrmPlusAction;
import com.nonesole.persistence2.sql.define.IPojoAction;
import com.nonesole.persistence2.sql.define.ISingleValueAction;
import com.nonesole.persistence2.sql.define.IVoidAction;
import com.nonesole.persistence2.sql.orm.DefaultOrmAction;
import com.nonesole.persistence2.sql.orm.DefaultOrmPlusAction;

/**
 * As the entrance of Persistence2 framework, it provides many persistence 
 * Interfaces to choose by developer.<br/><br/>
 * SQL sort:<br/>
 * DML: IOrmAction, IOrmPlusAction, IVoidAction.<br/>
 * DQL: IOrmAction, IOrmPlusAction, ISingleValueAction, IPojoAction, 
 *      IArrayValueAction, IListValueAction, IMapAction, IDyadicArrayValueAction,
 *      IDyadicListValueAction<br/><br/>
 * Relationship: <br/>
 * ORM: IOrmAction.<br/>
 * Customize ORM: IOrmPlusAction<br>
 * Customize SQL-MAPPING: ISingleValueAction, IPojoAction, 
 *      IArrayValueAction, IListValueAction, IMapAction, IDyadicArrayValueAction,
 *      IDyadicListValueAction<br/><br/>
 *  
 * @author jack lee
 * @version 1.0 - build on 2016-07-02
 */
public class PersistenceFactory {

	private static PersistenceFactory factory = null;
	private static String defaultDataSourceName = null;
	static {
		factory = new PersistenceFactory();
	}
	private PersistenceFactory(){
		defaultDataSourceName = MultiConnectionPool.getInstance().getDefaultDataSourceName();
	}
	
	public static PersistenceFactory getInstance(){
		return factory;
	}
	
	public final Connection getDefaultConnection() throws OperationsException{
		if ( defaultDataSourceName != null )
			return MultiConnectionPool.getInstance().getConnection(defaultDataSourceName);
		else
			throw new OperationsException("There is no default datasource.");
	}
	
	/**
	 * ThreadLocal for IOrmAction
	 */
	private static final ThreadLocal<IOrmAction> ormThread 
		= new ThreadLocal<IOrmAction>(); 
	
	/**
	 * ThreadLocal for IOrmPlusAction
	 */
	private static final ThreadLocal<IOrmPlusAction> ormplusThread 
		= new ThreadLocal<IOrmPlusAction>(); 
	
	/**
	 * Get IOrmAction Object
	 * @return IOrmAction
	 */
	public IOrmAction getOrmAction(){
		return this.getOrmAction(defaultDataSourceName);
	}
	
	/**
	 * Get IOrmAction Object
	 * @return IOrmAction
	 */
	public IOrmAction getOrmAction(String dataSourceName){
		IOrmAction ormAction = ormThread.get();
		if ( null == ormAction ) {
			ormAction = new DefaultOrmAction(dataSourceName);
			ormThread.set(ormAction);
		}
		return ormAction;
	}
	
	/**
	 * Get IOrmPlusAction Object
	 * @return IOrmAction
	 */
	public IOrmPlusAction getOrmPlusAction(){
		return this.getOrmPlusAction(defaultDataSourceName);
	}
	
	/**
	 * Get IOrmPlusAction Object
	 * @return IOrmAction
	 */
	public IOrmPlusAction getOrmPlusAction(String dataSourceName){
		IOrmPlusAction ormplusAction = ormplusThread.get();
		if ( null == ormplusAction ) {
			ormplusAction = new DefaultOrmPlusAction(dataSourceName);
			ormplusThread.set(ormplusAction);
		}
		return ormplusAction;
	}
	
	/**
	 *ThreadLocal for ISingleValueAction
	 */
	private static final ThreadLocal<ISingleValueAction> singleValueThread 
		= new ThreadLocal<ISingleValueAction>(); 
	
	/**
	 * Get ISingleValueAction Object.<br/>
	 * The objects of ISingleValueAction provide methods of single value return, like int,long,String,Object...
	 * @return ISingleValueAction
	 */
	public ISingleValueAction getSingleValueAction(){
		return this.getSingleValueAction(defaultDataSourceName);
	}
	
	/**
	 * Get ISingleValueAction Object.<br/>
	 * The objects of ISingleValueAction provide methods of single value return, like int,long,String,Object...
	 * @return ISingleValueAction
	 */
	public ISingleValueAction getSingleValueAction(String dataSourceName){
		ISingleValueAction singleValueAction = singleValueThread.get();
		if ( null == singleValueAction ) {
			singleValueAction = new DefaultSingleValueAction(dataSourceName);
			singleValueThread.set(singleValueAction);
		}
		return singleValueAction;
	}
	
	/**
	 *ThreadLocal for IPojoAction
	 */
	private static final ThreadLocal<IPojoAction<?>> pojoValueThread 
		= new ThreadLocal<IPojoAction<?>>(); 
	
	/**
	 * Get IPojoAction object<br>
	 * The objects of IPojoAction provide methods of pojo return, like pojo or list of pojo.
	 * @return IPojoAction<?>
	 */
	public IPojoAction<?> getPojoAction(){
		return this.getPojoAction(defaultDataSourceName);
	}
	
	/**
	 * Get IPojoAction object<br>
	 * The objects of IPojoAction provide methods of pojo return, like pojo or list of pojo.
	 * @return IPojoAction<?>
	 */
	@SuppressWarnings("rawtypes")
	public IPojoAction<?> getPojoAction(String dataSourceName){
		IPojoAction<?> pojoValueAction = pojoValueThread.get();
		if ( null == pojoValueAction ) {
			pojoValueAction = new DefaultPojoAction(dataSourceName);
			pojoValueThread.set(pojoValueAction);
		}
		return pojoValueAction;
	}

	/**
	 *ThreadLocal for IArrayValueAction
	 */
	private static final ThreadLocal<IArrayValueAction> arrayValueThread 
		= new ThreadLocal<IArrayValueAction>(); 
	
	/**
	 * Get IArrayValueAction object<br>
	 * The objects of IArrayValueAction provide methods of array return.
	 * @return IArrayValueAction
	 */
	public IArrayValueAction getArrayValueAction(){
		return this.getArrayValueAction(defaultDataSourceName);
	}

	/**
	 * Get IArrayValueAction object<br>
	 * The objects of IArrayValueAction provide methods of array return.
	 * @return IArrayValueAction
	 */
	public IArrayValueAction getArrayValueAction(String dataSourceName){
		IArrayValueAction arrayValueAction = arrayValueThread.get();
		if ( null == arrayValueAction ) {
			arrayValueAction = new DefaultArrayValueAction(dataSourceName);
			arrayValueThread.set(arrayValueAction);
		}
		return arrayValueAction;
	}
	
	/**
	 *ThreadLocal for IListValueAction
	 */
	private static final ThreadLocal<IListValueAction> listValueThread 
		= new ThreadLocal<IListValueAction>(); 
	
	/**
	 * Get IListValueAction object<br>
	 * The objects of IListValueAction provide methods of list return.
	 * @return IListValueAction
	 */	
	public IListValueAction getListValueAction(){
		return this.getListValueAction(defaultDataSourceName);
	}

	/**
	 * �õ������Զ���ķ���ֵΪ���ϵ�Action����
	 * Get IListValueAction object<br>
	 * The objects of IListValueAction provide methods of list return.
	 * @return IListValueAction
	 */	
	public IListValueAction getListValueAction(String dataSourceName){
		IListValueAction listValueAction = listValueThread.get();
		if ( null == listValueAction ) {
			listValueAction = new DefaultListValueAction(dataSourceName);
			listValueThread.set(listValueAction);
		}
		return listValueAction;
	}
	
	/**
	 *ThreadLocal�������ڹ�������Զ���ķ���ֵΪMap����Ĳ�����֧�ֶ��̲߳���
	 *ThreadLocal for IMapAction
	 */
	private static final ThreadLocal<IMapAction> mapValueThread 
		= new ThreadLocal<IMapAction>(); 
	
	/**
	 *ThreadLocal�������ڹ�������Զ���ķ���ֵΪvoid����Ĳ�����֧�ֶ��̲߳���
	 * ThreadLocal for IVoidAction
	 */
	private static final ThreadLocal<IVoidAction> voidValueThread 
		= new ThreadLocal<IVoidAction>(); 
	
	/**
	 * �õ������Զ���ķ���ֵΪMap�����Action����<br>
	 * Get IMapAction object<br>
	 * The objects of IMapAction provide methods of map return or list of map return.
	 * @return IOtherCustomAction
	 */	
	public IMapAction getMapAction(){
		return this.getMapAction(defaultDataSourceName);
	}

	/**
	 * �õ������Զ���ķ���ֵΪMap�����Action����<br>
	 * Get IMapAction object<br>
	 * The objects of IMapAction provide methods of map return or list of map return.
	 * @return IOtherCustomAction
	 */	
	public IMapAction getMapAction(String dataSourceName){
		IMapAction mapValueAction = mapValueThread.get();
		if ( null == mapValueAction ) {
			mapValueAction = new DefaultMapAction(dataSourceName);
			mapValueThread.set(mapValueAction);
		}
		return mapValueAction;
	}
	
	/**
	 *ThreadLocal�������ڹ�������Զ���ķ���ֵΪ��ά����Ĳ�����֧�ֶ��̲߳���
	 *ThreadLocal for IDyadicArrayValueAction
	 */
	private static final ThreadLocal<IDyadicArrayValueAction> dyadicArrayValueThread 
		= new ThreadLocal<IDyadicArrayValueAction>(); 
	
	/**
	 * �õ������Զ���ķ���ֵΪ�����Action����<br>
	 * Get IDyadicArrayValueAction object<br>
	 * The objects of IDyadicArrayValueAction provide methods of dyaic array return.
	 * @return IDyadicArrayValueAction
	 */	
	public IDyadicArrayValueAction getDyadicArrayValueAction(){
		return this.getDyadicArrayValueAction(defaultDataSourceName);
	}

	/**
	 * �õ������Զ���ķ���ֵΪ�����Action����<br>
	 * Get IDyadicArrayValueAction object<br>
	 * The objects of IDyadicArrayValueAction provide methods of dyaic array return.
	 * @return IDyadicArrayValueAction
	 */	
	public IDyadicArrayValueAction getDyadicArrayValueAction(String dataSourceName){
		IDyadicArrayValueAction dyadicArrayValueAction = dyadicArrayValueThread.get();
		if ( null == dyadicArrayValueAction ) {
			dyadicArrayValueAction = new DefaultDyadicArrayValueAction(dataSourceName);
			dyadicArrayValueThread.set(dyadicArrayValueAction);
		}
		return dyadicArrayValueAction;
	}
	
	/**
	 *ThreadLocal�������ڹ�������Զ���ķ���ֵΪ��ά���ϵĲ�����֧�ֶ��̲߳���
	 *ThreadLocal for IDyadicListValueAction
	 */
	private static final ThreadLocal<IDyadicListValueAction> dyadicListValueThread 
		= new ThreadLocal<IDyadicListValueAction>(); 
	
	/**
	 * �õ������Զ���ķ���ֵΪ��ά���ϵ�Action����<br>
	 * Get IDyadicListValueAction object<br>
	 * The objects of IDyadicListValueAction provide methods of dyaic list return.
	 * @return IDyadicListValueAction
	 */	
	public IDyadicListValueAction getDyadicListValueAction(){
		return this.getDyadicListValueAction(defaultDataSourceName);
	}
	
	/**
	 * �õ������Զ���ķ���ֵΪ��ά���ϵ�Action����<br>
	 * Get IDyadicListValueAction object<br>
	 * The objects of IDyadicListValueAction provide methods of dyaic list return.
	 * @return IDyadicListValueAction
	 */	
	public IDyadicListValueAction getDyadicListValueAction(String dataSourceName){
		IDyadicListValueAction dyadicListValueAction = dyadicListValueThread.get();
		if ( null == dyadicListValueAction ) {
			dyadicListValueAction = new DefaultDyadicListValueAction(dataSourceName);
			dyadicListValueThread.set(dyadicListValueAction);
		}
		return dyadicListValueAction;
	}
	
	/**
	 * �õ������Զ���ķ���ֵΪvoid��Action����<br>
	 * Get IVoidAction object
	 * @return IOtherCustomAction
	 */	
	public IVoidAction getVoidAction(){
		return this.getVoidAction(defaultDataSourceName);
	}

	/**
	 * �õ������Զ���ķ���ֵΪvoid��Action����<br>
	 * Get IVoidAction object
	 * @return IOtherCustomAction
	 */	
	public IVoidAction getVoidAction(String dataSourceName){
		IVoidAction action = voidValueThread.get();
		if ( null == action ) {
			action = new DefaultVoidAction(dataSourceName);
			voidValueThread.set(action);
		}
		return action;
	}
	
}
