/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.basic;

import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.config.CommonTools;
import com.nonesole.persistence2.config.bean.OSMObject;
import com.nonesole.persistence2.config.bean.OutputMapping;
import com.nonesole.persistence2.connection.exception.OperationsException;
import com.nonesole.persistence2.sql.ConfigLoader;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;
import com.nonesole.persistence2.sql.tools.ReflectToolkit;

/**
 * This is the base of default classes, it provides public 
 * and protected methods for other's calling. 
 * @author jack lee
 * @version 1.0
 */
public class DefaultBaseAction {

	private Logger logger = Logger.getInstance();
	
	protected boolean isPreparedStatement = true;
	
	public DefaultBaseAction(boolean isPreparedStatement){
		this.isPreparedStatement = isPreparedStatement;
	}
	
	/**
	 * Get value frome get method
	 * @param field Field Object 
	 * @param obj Object needs do with
	 * @throws ExecuteSQLException
	 * @throws SecurityException 
	 * @throws NoSuchMethodException 
	 * @throws InvocationTargetException 
	 * @throws IllegalArgumentException 
	 * @throws IllegalAccessException 
	 * */
	public static Object invoke(Field field,Object obj) 
			throws ExecuteSQLException, NoSuchMethodException, SecurityException, IllegalAccessException, IllegalArgumentException, InvocationTargetException{
		if ( CommonTools.isNull(field) ) {
			throw new ExecuteSQLException("Field is null.");
		}
		if ( CommonTools.isNull(obj) ) {
			throw new ExecuteSQLException("Object is null.");
		}
		
		int modifier = field.getModifiers();
		if ( Modifier.isFinal(modifier) ||
				(Modifier.isStatic(modifier) && Modifier.isPublic(modifier) ) ) {
			return null;
		} 
		Class<?> fieldType = field.getType();
		String fieldName = field.getName();
		Method m = null;
		Class<?> classType = obj.getClass();
		StringBuilder sb = new StringBuilder();
		if ( fieldType == boolean.class ) {
			//if some method return boolean then needs special deal with 
		    if ( "is".equals(fieldName.substring(0,2)) )//isXXX method
		    	sb.append("is").
				append(fieldName.substring(2, fieldName.length()));
		    else//field name do not contain "is"
				sb.append("is").
				append(fieldName.substring(0,1).toUpperCase()).
				append(fieldName.substring(1, fieldName.length()));
		
		} else//non-boolean-return method
			sb.append("get").
				append(fieldName.substring(0,1).toUpperCase()).
				append(fieldName.substring(1, fieldName.length()));
		m = classType.getMethod(sb.toString(),new Class[]{});
		return m.invoke(obj, new Object[]{});
	}
			
	/**
	 * Check field whether or not needs '' around it's value
	 * @param fieldType  field type
	 * @param sb string which needs join
	 * @param value value which will join the string
	 * @throws ExecuteSQLException
	 * */
	public static void handleCharacter(
			Class<?> fieldType,StringBuilder sb,Object value) 
	throws ExecuteSQLException{
		if ( CommonTools.isNull(fieldType) ) {
			throw new ExecuteSQLException("Fieldtype is null.");
		}
		if ( CommonTools.isNull(sb) ) {
			throw new ExecuteSQLException("StringBuilder is null.");
		}
		if ( CommonTools.isNull(value) ) {
			throw new ExecuteSQLException("Value is null.");
		}
		
		if ( fieldType == String.class 
				|| fieldType == Date.class 
				|| fieldType == Time.class
				|| fieldType == Timestamp.class 
				|| fieldType == Character.class
				|| fieldType == char.class)	{
			sb.append("'").append(value).append("'");
		} else if ( boolean.class == fieldType || Boolean.class == fieldType
				 	|| int.class == fieldType || Integer.class == fieldType 
				 	|| long.class == fieldType || Long.class == fieldType
				 	|| float.class == fieldType || Float.class == fieldType
				 	|| double.class == fieldType || Double.class == fieldType
				 	|| short.class == fieldType || Short.class == fieldType
				 	|| Byte.class == fieldType || byte.class == fieldType
				 	|| BigDecimal.class == fieldType 
				 	|| BigInteger.class == fieldType ) {
			sb.append(value);
		} else if ( fieldType.isArray() ) {
			Class<?> arrayType = fieldType.getComponentType();
			int length = Array.getLength(value);
			if ( arrayType.isPrimitive() ) {
				//if value is primitive
				for ( int i = 0 ; i < length ; i++ ) {
					sb.append(Array.get(value, i));
				}
			} else {
				for ( int i = 0 ; i < length ; i++ ) {
					sb.append(Array.get(value, i).toString());
				}
			}
			
		}else {
			sb.append(value.toString());
		}
	}
	
	/**
	 * Array to vector
	 * @param input
	 * @return
	 */
	public static Vector<String> arrayToVector(String[] input) {
		Vector<String> key = new Vector<String>();
		if ( null != input ) {
			for ( String k : input ) {
				key.add(k);
			}
		}
		return key;
	}
	
	/**
	 * Set values to '?' of where
	 * @param st PreparedStatement object
	 * @param values to '?'
	 * @param type of values
	 * @param startPosition start position of values 
	 * */
	protected void setValueToWhere(
			PreparedStatement st,
			Vector<Object> values,
			Vector<Class<?>> fTypes,
			int startPosition)
	throws ExecuteSQLException {
		DefaultBasicPreparedAction.setValues(st, values, fTypes, null,startPosition);
	}
	
	/**
	 * Insert data but not commit.
	 * @param obj
	 * @param conn
	 * @return obj
	 * @throws InvocationTargetException 
	 * @throws IllegalArgumentException 
	 * @throws IllegalAccessException 
	 * @throws SecurityException 
	 * @throws NoSuchMethodException 
	 */
	protected Object insertWithoutCommit(Object obj, Connection conn, boolean isPrepared)
			throws ExecuteSQLException, NoSuchMethodException, SecurityException, IllegalAccessException, IllegalArgumentException, InvocationTargetException {
		if ( CommonTools.isNull(obj) ) {
			throw new ExecuteSQLException("Object is null.");
		}
		if ( CommonTools.isNull(conn) ) {
			throw new ExecuteSQLException("Connection is null.");
		}
		
		OSMObject osm = ConfigLoader.getInstance()
									.getObjectSqlMapping(obj.getClass().getName());
		if ( null == osm ) {
			throw new ExecuteSQLException("ObjectSqlMapping is null.");
		}
		Field[] fields = getFields(obj);
		if ( isPrepared ) {			
			String name;
			Class<?> type = null;
			StringBuilder dbField = new StringBuilder();
			StringBuilder values = new StringBuilder(); 
			Vector<Object> v = new Vector<Object>();
			Vector<Class<?>> fTypes = new Vector<Class<?>>();
			
			for( Field field : fields ){
				name = field.getName();
				type = field.getType();
				int modifier = field.getModifiers();
				if ( Modifier.isFinal(modifier) ||
						(Modifier.isStatic(modifier) && Modifier.isPublic(modifier) ) ) {
					continue;
				} 
				dbField.append(name).append(",");
				v.add(invoke(field, obj));
				fTypes.add(type);
				values.append("?,");			
			} 
			int point = dbField.lastIndexOf(",");
			dbField.delete(point, point+1);
			point = values.lastIndexOf(",");
			values.delete(point, point+1);
					
			StringBuilder sql = new StringBuilder();
			
			sql.append("insert into")
				.append(" ")
				.append(osm.getTableName())
				.append("(")
				.append(dbField.toString())
				.append(") values (")
				.append(values.toString())
				.append(")");
			
			logger.debug("Orm insert : " + sql.toString(), null);
			//Get Statement
			PreparedStatement st = DefaultBasicPreparedAction.getPreparedStatement(conn,sql.toString());
			Vector<Integer> blobPosition = new Vector<Integer>();
			DefaultBasicPreparedAction.setValues(st,v,fTypes,blobPosition,0);
			//if blob is exist,deal with it
			DefaultBasicPreparedAction.executeSave(st, obj);
			DefaultBasicPreparedAction.saveBlob(conn,obj,blobPosition,v);
		} else {
			String sql = this.transferToInsertSql(obj);
			try {
				Logger.getInstance().info("Statement. " + sql, null);
				Statement st = conn.createStatement();
				st.execute(sql);
				DefaultBaseAction.getGeneratedKeys(obj, st);
				st.close();
				st = null;
			} catch (SQLException e) {
				DefaultBaseAction.connRelease(conn, null, null);
				throw new ExecuteSQLException(e);
			}
		}
		
		return obj;
	}

	/**
	 * Update data but not commit.
	 * @param obj
	 * @param conn
	 * @return obj
	 * @throws InvocationTargetException 
	 * @throws IllegalArgumentException 
	 * @throws IllegalAccessException 
	 * @throws SecurityException 
	 * @throws NoSuchMethodException 
	 */
	protected Object updateWithoutCommit(Object obj, Connection conn, boolean isPrepared)
			throws ExecuteSQLException, NoSuchMethodException, SecurityException, IllegalAccessException, IllegalArgumentException, InvocationTargetException {
		if ( CommonTools.isNull(obj) ) {
			throw new ExecuteSQLException("Object is null.");
		}
		if ( CommonTools.isNull(conn) ) {
			throw new ExecuteSQLException("Connection is null.");
		}
		
		OSMObject osm = ConfigLoader.getInstance()
									.getObjectSqlMapping(obj.getClass().getName());
		if ( null == osm ) {
			throw new ExecuteSQLException("ObjectSqlMapping is null.");
		}
		Field[] fields = getFields(obj);
		if ( isPrepared ) {
			String name;
			Class<?> type = null;
			StringBuilder set = new StringBuilder();
			StringBuilder where = new StringBuilder(); 
			StringBuilder temp = null;
			Object value = null;
			Vector<Object> values = new Vector<Object>();
			Vector<Class<?>> fTypes = new Vector<Class<?>>();
			Vector<String> key = arrayToVector(osm.getPrimaryKeys());
			Vector<Object> primeryKeyValues = new Vector<Object>();
			Vector<Class<?>> primeryKeyTypes = new Vector<Class<?>>();
			
			for( Field field : fields ){
				name = field.getName();
				type = field.getType();	
				int modifier = field.getModifiers();
				if ( Modifier.isFinal(modifier) ||
						(Modifier.isStatic(modifier) && Modifier.isPublic(modifier) ) ) {
					continue;
				} 	
				temp = new StringBuilder();
				value = invoke(field, obj);
				values.add(value);
				fTypes.add(type);
				temp.append(name).append("=").append("?");
				set.append(temp).append(",");
				if ( key.contains(name) ) {
					where.append(temp).append(" and ");
					//Save prepared parameter
					primeryKeyValues.add(value);
					primeryKeyTypes.add(type);
				}
							
			} 
			int point = set.lastIndexOf(",");
			set.delete(point, point+1);
			point = where.lastIndexOf(" and ");
			where.delete(point, point+6);
			
			StringBuilder sql = new StringBuilder();
			sql.append("update")
				.append(" ")
				.append(osm.getTableName())
				.append(" ")
				.append("set")
				.append(" ")
				.append(set)
				.append(" ")
				.append("where")
				.append(" ")
				.append(where);
			logger.debug("Orm update : " + sql.toString(), null);
			PreparedStatement st = DefaultBasicPreparedAction.getPreparedStatement(conn,sql.toString());
			Vector<Integer> blobPosition = new Vector<Integer>(); 
			DefaultBasicPreparedAction.setValues(st,values,fTypes,blobPosition,0);
			setValueToWhere(st, primeryKeyValues, primeryKeyTypes, values.size()+1);
			DefaultBasicPreparedAction.execute(st);	
			DefaultBasicPreparedAction.saveBlob(conn,obj,blobPosition,values);
		} else {
			String sql = this.transferToUpdateSql(obj);
			try {
				Logger.getInstance().info("Statement. " + sql, null);
				conn.createStatement().execute(sql);
			} catch (SQLException e) {
				DefaultBaseAction.connRelease(conn, null, null);
				throw new ExecuteSQLException(e);
			}
		}
		
		return obj;
	}

	/**
	 * Delete data but not commit.
	 * @param obj
	 * @param conn
	 * @throws ExecuteSQLException
	 * @throws InvocationTargetException 
	 * @throws IllegalArgumentException 
	 * @throws IllegalAccessException 
	 * @throws SecurityException 
	 * @throws NoSuchMethodException 
	 */
	protected void deleteWithoutCommit(Object obj, Connection conn, boolean isPrepared) 
	throws ExecuteSQLException, NoSuchMethodException, SecurityException, IllegalAccessException, IllegalArgumentException, InvocationTargetException {
		if ( CommonTools.isNull(obj) ) {
			throw new ExecuteSQLException("Object is null.");
		}
		if ( CommonTools.isNull(conn) ) {
			throw new ExecuteSQLException("Connection is null.");
		}
		
		OSMObject osm = ConfigLoader.getInstance()
									.getObjectSqlMapping(obj.getClass().getName());
		if ( null == osm ) {
			throw new ExecuteSQLException("ObjectSqlMapping is null.");
		}
		Field[] fields = getFields(obj);
		
		if ( isPrepared ) {
			StringBuilder where = new StringBuilder(); 
			Vector<String> key = arrayToVector(osm.getPrimaryKeys());
			Vector<Object> primeryKeyValues = new Vector<Object>();
			Vector<Class<?>> primeryKeyTypes = new Vector<Class<?>>();

			int primeryKeyCount = 0;
			for( Field field : fields ){
				if ( key.contains(field.getName()) ) {
					if ( primeryKeyCount > 0 ) {
						where.append(" and ");
					}
					where.append(field.getName()).append("=").append("?");
					//Save prepared parameter
					primeryKeyValues.add(invoke(field, obj));
					primeryKeyTypes.add(field.getType());
					primeryKeyCount++;
				}
							
			} 
			
			StringBuilder sql = new StringBuilder();
			sql.append("delete from")
				.append(" ")
				.append(osm.getTableName())
				.append(" ")
				.append("where")
				.append(" ")
				.append(where);
			logger.debug("Orm delete : " + sql.toString(), null);
			PreparedStatement st = DefaultBasicPreparedAction.getPreparedStatement(conn,sql.toString());
			setValueToWhere(st, primeryKeyValues, primeryKeyTypes, primeryKeyCount);
			DefaultBasicPreparedAction.execute(st);	
		} else {
			String sql = this.transferToDeleteSql(obj);
			try {
				Logger.getInstance().info("Statement. " + sql, null);
				conn.createStatement().execute(sql);
			} catch (SQLException e) {
				DefaultBaseAction.connRelease(conn, null, null);
				throw new ExecuteSQLException(e);
			}
		}
		
	}
	
	/**
	 * Transform object to delete sql
	 * @param obj Object needs transform
	 * @return String
	 * @throws InvocationTargetException 
	 * @throws IllegalArgumentException 
	 * @throws IllegalAccessException 
	 * @throws SecurityException 
	 * @throws NoSuchMethodException 
	 * */
	protected String transferToDeleteSql(Object obj) throws ExecuteSQLException, NoSuchMethodException, SecurityException, IllegalAccessException, IllegalArgumentException, InvocationTargetException{
		if ( CommonTools.isNull(obj) ) {
			throw new ExecuteSQLException("Object is null.");
		}
		
		OSMObject osm = ConfigLoader.getInstance()
				.getObjectSqlMapping(obj.getClass().getName());
		if ( null == osm ) {
			throw new ExecuteSQLException("ObjectSqlMapping is null.");
		}
		
		Class<?> c = obj.getClass();
		java.lang.reflect.Field[] fields = c.getDeclaredFields();
		if ( fields == null )
			throw new ExecuteSQLException("Fields is null.");
					
		String name;
		Object value;
		Class<?> type = null;
		StringBuilder where = new StringBuilder(); 
		StringBuilder temp = null;
		
		Vector<String> key = arrayToVector(osm.getPrimaryKeys());
		for( Field field : fields ){
			name = field.getName();
			type = field.getType();	
			int modifier = field.getModifiers();
			if ( Modifier.isFinal(modifier) ||
					(Modifier.isStatic(modifier) && Modifier.isPublic(modifier) ) ) {
				//no set and get method if field is public + static 
				continue;
			} 			
			temp = new StringBuilder();
			value = invoke(field, obj);
			temp.append(name).append("=");
			handleCharacter(type,temp,value);
			
			if ( key.contains(name) ) {
				where.append(temp).append(" and ");
			}
						
		} 
		int point = where.lastIndexOf(" and ");
		where.delete(point, point+6);
		
		StringBuilder sql = new StringBuilder();
		
		sql.append("delete from")
			.append(" ")
			.append(osm.getTableName())
			.append(" ")
			.append("where")
			.append(" ")
			.append(where);
		
		return sql.toString();
	}
	
	/**
	 * Transform object to update sql
	 * @param obj Object needs transform
	 * @return String
	 * @throws InvocationTargetException 
	 * @throws IllegalArgumentException 
	 * @throws IllegalAccessException 
	 * @throws SecurityException 
	 * @throws NoSuchMethodException 
	 * */
	protected String transferToUpdateSql(Object obj) throws ExecuteSQLException, NoSuchMethodException, SecurityException, IllegalAccessException, IllegalArgumentException, InvocationTargetException{
		if ( CommonTools.isNull(obj) ) {
			throw new ExecuteSQLException("Object is null.");
		}
		
		OSMObject osm = ConfigLoader.getInstance()
				.getObjectSqlMapping(obj.getClass().getName());
		if ( null == osm ) {
			throw new ExecuteSQLException("ObjectSqlMapping is null.");
		}		
		
		Class<?> c = obj.getClass();
		java.lang.reflect.Field[] fields = c.getDeclaredFields();
		if ( fields == null )
			throw new ExecuteSQLException("Fields is null.");
					
		String name;
		Object value;
		Class<?> type = null;
		StringBuilder set = new StringBuilder();
		StringBuilder where = new StringBuilder(); 
		StringBuilder temp = null;
		
		Vector<String> key = arrayToVector(osm.getPrimaryKeys());
		for( Field field : fields ){
			name = field.getName();
			type = field.getType();	
			int modifier = field.getModifiers();
			if ( Modifier.isFinal(modifier) ||
					(Modifier.isStatic(modifier) && Modifier.isPublic(modifier) ) ) {
				continue;
			} 
			temp = new StringBuilder();
			value = invoke(field, obj);
			temp.append(name).append("=");
			handleCharacter(type,temp,value);
			
			set.append(temp).append(",");
			if ( key.contains(name) ) {
				where.append(temp).append(" and ");
			}
						
		} 
		int point = set.lastIndexOf(",");
		set.delete(point, point+1);
		point = where.lastIndexOf(" and ");
		where.delete(point, point+6);
		
		StringBuilder sql = new StringBuilder();
		
		sql.append("update")
			.append(" ")
			.append(osm.getTableName())
			.append(" ")
			.append("set")
			.append(" ")
			.append(set)
			.append(" ")
			.append("where")
			.append(" ")
			.append(where);
		
		return sql.toString();
	}
	 
	/**
	 * Transform object to insert sql
	 * @param obj Object needs transform
	 * @return String
	 * @throws InvocationTargetException 
	 * @throws IllegalArgumentException 
	 * @throws IllegalAccessException 
	 * @throws SecurityException 
	 * @throws NoSuchMethodException 
	 * */
	protected String transferToInsertSql(Object obj) throws ExecuteSQLException, NoSuchMethodException, SecurityException, IllegalAccessException, IllegalArgumentException, InvocationTargetException{
		if ( CommonTools.isNull(obj) ) {
			throw new ExecuteSQLException("Object is null.");
		}
		
		OSMObject osm = ConfigLoader.getInstance()
				.getObjectSqlMapping(obj.getClass().getName());
		if ( null == osm ) {
			throw new ExecuteSQLException("ObjectSqlMapping is null.");
		}
		
		Class<?> c = obj.getClass();
		java.lang.reflect.Field[] fields = c.getDeclaredFields();
		if ( fields == null )
			throw new ExecuteSQLException("Fields is null.");
		
		String name;
		Object value;
		Class<?> type = null;
		StringBuilder dbField = new StringBuilder();
		StringBuilder values = new StringBuilder();
		for( Field field : fields ){
			name = field.getName();
			type = field.getType();
			int modifier = field.getModifiers();
			if ( Modifier.isFinal(modifier) ||
					(Modifier.isStatic(modifier) && Modifier.isPublic(modifier) ) ) {
				continue;
			} 
			dbField.append(name).append(",");
			value = invoke(field, obj);
			handleCharacter(type,values,value);
			values.append(",");			
		} 
		int point = dbField.lastIndexOf(",");
		dbField.delete(point, point+1);
		point = values.lastIndexOf(",");
		values.delete(point, point+1);
		
		//sql
		StringBuilder sql = new StringBuilder();
		sql.append("insert into")
			.append(" ")
			.append(osm.getTableName())
			.append("(")
			.append(dbField.toString())
			.append(") values (")
			.append(values.toString())
			.append(")");
		
		return sql.toString();
	}
	
	/**
	 * Commit connection
	 * @param conn
	 * @param commit
	 * @throws ExecuteSQLException
	 */
	public static void commitConnection(Connection conn,boolean commit) 
			throws ExecuteSQLException {
		if ( commit ) {
			try {
				if ( !conn.getAutoCommit() ) {
					conn.commit();
				}
				conn.close();
				conn = null;
			} catch (SQLException e) {
				throw new ExecuteSQLException(e);
			} 
		} else {
			try {
				conn.setAutoCommit(false);
			} catch (SQLException e) {
				throw new ExecuteSQLException(e);
			}
		}
	}
	
	/**
	 * Deal with injection attacks
	 * @param sql
	 */
	public static void preventInjectionAttacks(String sql) 
			throws ExecuteSQLException {
		sql = sql.replaceAll("'", "''");
	}
	
	/**
	 * Query
	 * @param sql
	 * @param clazz
	 * @param conn
	 * @return
	 * @throws ExecuteSQLException
	 */
	protected List<Object> query(String sql, Class<?> clazz, Connection conn)
			throws ExecuteSQLException {
		if ( CommonTools.isNull(sql) ) {
			throw new ExecuteSQLException("SQL is null.");
		}
		if ( CommonTools.isNull(conn) ) {
			throw new ExecuteSQLException("Connection is null.");
		}
		if ( CommonTools.isNull(clazz) ) {
			throw new ExecuteSQLException("Class is null.");
		}
		preventInjectionAttacks(sql);
		List<Object> l = new ArrayList<Object>();
		Statement stmt = null;
		ResultSet rs = null;
		try {
			Logger.getInstance().info("Statement. " + sql, null);
			stmt = conn.createStatement();
			rs = stmt.executeQuery(sql);
			Object obj = clazz.newInstance();
			while( rs.next()) {
				obj = clazz.newInstance();
				//transform ResultSet to object
				ReflectToolkit.parseResultSetToObject(rs, obj);
				l.add(obj);
			}
		} catch (SQLException e) {
			connRollback(conn, e);
		} catch (Exception e) {
			throw new ExecuteSQLException("Executing sql is failure.",e.fillInStackTrace());
		} finally {
			connRelease(conn, rs, stmt);
		}
		return l;
	}

	
	/**
	 * Execute SQL.
	 * @param sql
	 * @param conn
	 * @throws ExecuteSQLException
	 */
	protected void execute(String sql, Connection conn) throws ExecuteSQLException {
		if ( CommonTools.isNull(sql) ) {
			throw new ExecuteSQLException("SQL is null.");
		}
		if ( CommonTools.isNull(conn) ) {
			throw new ExecuteSQLException("Connection is null.");
		}
		preventInjectionAttacks(sql);
		Statement stmt = null;
		try {
			Logger.getInstance().info("Statement. " + sql, null);
			stmt = (Statement) conn.createStatement();
			stmt.execute(sql);
		} catch (SQLException e) {
			connRollback(conn, e);
		} catch (Exception e) {
			throw new ExecuteSQLException(
					"Executing sql is failure.",e.fillInStackTrace());
		} finally {
			connRelease(conn, null, stmt);
		}
	}
	
	/**
	 * Execute SQL.
	 * @param sql
	 * @param conn
	 * @throws ExecuteSQLException
	 */
	protected void executeSave(String sql, Connection conn, Object obj) throws ExecuteSQLException {
		if ( CommonTools.isNull(sql) ) {
			throw new ExecuteSQLException("SQL is null.");
		}
		if ( CommonTools.isNull(conn) ) {
			throw new ExecuteSQLException("Connection is null.");
		}
		preventInjectionAttacks(sql);
		Statement stmt = null;
		try {
			Logger.getInstance().info("Statement. " + sql, null);
			stmt = (Statement) conn.createStatement();
			stmt.execute(sql);
			DefaultBaseAction.getGeneratedKeys(obj, stmt);
		} catch (SQLException e) {
			connRollback(conn, e);
		} catch (Exception e) {
			throw new ExecuteSQLException(
					"Executing sql is failure.",e.fillInStackTrace());
		} finally {
			connRelease(conn, null, stmt);
		}
	}
	
	/**
	 * Execute SQL but not commit.
	 * @param sql
	 * @param conn
	 * @throws ExecuteSQLException
	 */
	public static void executeNotCommit(String sql, Connection conn) throws ExecuteSQLException {
		if ( CommonTools.isNull(sql) ) {
			throw new ExecuteSQLException("SQL is null.");
		}
		if ( CommonTools.isNull(conn) ) {
			throw new ExecuteSQLException("Connection is null.");
		}
		preventInjectionAttacks(sql);
		Statement stmt = null;
		try {
			Logger.getInstance().info("Statement. " + sql, null);
			stmt = (Statement) conn.createStatement();
			stmt.execute(sql);
		} catch (SQLException e) {
			connRollback(conn, e);
		} 
	}
		
	/**
	 * Query and return dyadic list.
	 * @param sql
	 * @param clazz
	 * @param conn
	 * @return
	 * @throws ExecuteSQLException
	 */
	public static List<List<Object>> queryReturnDyadicList(
			String sql, Class<?> clazz, Connection conn,
			List<OutputMapping> mapping)
			throws ExecuteSQLException {
		if ( CommonTools.isNull(sql) ) {
			throw new ExecuteSQLException("SQL is null.");
		}
		if ( CommonTools.isNull(conn) ) {
			throw new ExecuteSQLException("Connection is null.");
		}
		if ( CommonTools.isNull(clazz) ) {
			throw new ExecuteSQLException("Class is null.");
		}
		preventInjectionAttacks(sql);
		Statement stmt = null;
		ResultSet rs = null;
		List<List<Object>> list = new ArrayList<List<Object>>();
		List<Object> objs = null;
		try {
			stmt = conn.createStatement();
			rs = stmt.executeQuery(sql);
			String rsKey = null;
			Class<?> type = null;
			while ( rs.next()) {	
				objs = new ArrayList<Object>();
				for ( OutputMapping m : mapping ) {
					//type = Class.forName(m.getFiledType());
					//Logger.getInstance().info(m.getSourceName() + ";" + m.getTargetClass(), null);
					type = str2Type(m.getTargetClass());
					rsKey = m.getSourceName();
					objs.add(rs2Obj(type, rs, rsKey));
				}
				list.add(objs);
			}
		} catch (SQLException e) {
			connRollback(conn, e);
		} catch (Exception e) {
			throw new ExecuteSQLException(
					"Executing sql is failure.",e.fillInStackTrace());
		} finally {
			connRelease(conn, rs, stmt);
		}
		return list;
	}
	
	
	/**
	 * Transform ResultSet to Object.
	 * @param clazz 
	 * @param rs 
	 * @param rsKey
	 * @return Object
	 * @throws SQLException
	 */
	public static Object rs2Obj(Class<?> clazz,ResultSet rs,String rsKey) throws ExecuteSQLException{
		Object obj = null;
		try {
			if ( boolean.class == clazz || Boolean.class == clazz ) {
				obj = rs.getBoolean(rsKey);
			} else if ( int.class == clazz || Integer.class == clazz ) {
				obj = rs.getInt(rsKey);
			} else if ( long.class == clazz || Long.class == clazz ) {
				obj = rs.getLong(rsKey);
			} else if ( float.class == clazz || Float.class == clazz ) {
				obj = rs.getFloat(rsKey);
			} else if ( double.class == clazz || Double.class == clazz ) {
				obj = rs.getDouble(rsKey);
			} else if ( short.class == clazz || Short.class == clazz ) {
				obj = rs.getShort(rsKey);
			} else if ( Byte.class == clazz || byte.class == clazz ) {
				obj = rs.getByte(rsKey);
			} else if ( Character.class == clazz || char.class == clazz ) {
				obj = rs.getString(rsKey);
			} else if ( String.class == clazz ) {
				obj = rs.getString(rsKey);
			} else {
				obj = rs.getObject(1);
			}
		} catch (SQLException e) {
			throw new ExecuteSQLException("DefaultBaseAction:rs2Obj",e.fillInStackTrace());
		}
		return obj;
	}
	
	/**
	 * Transform ResultSet to Object.
	 * @param clazz
	 * @param rs 
	 * @param rsKey 
	 * @return Object
	 * @throws SQLException
	 */
	public static Object rs2Obj(Class<?> clazz,ResultSet rs,int point) throws ExecuteSQLException{
		Object obj = null;
		if ( point < 1 ) 
			point = 1;
		try {
			if ( boolean.class == clazz || Boolean.class == clazz ) {
				obj = rs.getBoolean(point);
			} else if ( int.class == clazz || Integer.class == clazz ) {
				obj = rs.getInt(point);
			} else if ( long.class == clazz || Long.class == clazz ) {
				obj = rs.getLong(point);
			} else if ( float.class == clazz || Float.class == clazz ) {
				obj = rs.getFloat(point);
			} else if ( double.class == clazz || Double.class == clazz ) {
				obj = rs.getDouble(point);
			} else if ( short.class == clazz || Short.class == clazz ) {
				obj = rs.getShort(point);
			} else if ( Byte.class == clazz || byte.class == clazz ) {
				obj = rs.getByte(point);
			} else if ( Character.class == clazz || char.class == clazz ) {
				obj = rs.getString(point);
			} else if ( String.class == clazz ) {
				obj = rs.getString(point);
			} else {
				obj = rs.getObject(point);
			}
		} catch (SQLException e) {
			throw new ExecuteSQLException("DefaultBaseAction:rs2Obj",e.fillInStackTrace());
		}
		return obj;
	}
	
	/**
	 * Transform String to object type
	 * @param str
	 * @return
	 * @throws ExecuteSQLException
	 * @throws ClassNotFoundException 
	 */
	public static Class<?> str2Type(String str) throws ExecuteSQLException{
		if ( null == str || "".equals(str) ) {
			throw new ExecuteSQLException("Input string is null, can not get Class.");
		}
		Class<?> type = null;
		if ( "int".equals(str) ) {
			type = int.class;
		} else if ( "short".equals(str) ) {
			type = short.class;
		} else if ( "long".equals(str) ) {
			type = long.class;
		} else if ( "float".equals(str) ) {
			type = float.class;
		} else if ( "double".equals(str) ){
			type = double.class;
		} else if ( "byte".equals(str) ){
			type = byte.class;
		} else if ( "boolean".equals(str) ){
			type = boolean.class;
		} else if ( "char".equals(str) ){
			type = Character.class;
		} else if ( "String".equals(str) ){
			type = String.class;
		} else {
			try {
				type = Class.forName(str);
			} catch (ClassNotFoundException e) {
				throw new ExecuteSQLException(e.fillInStackTrace());
			}
		}
		return type;
	}
	
	/**
	 * Rollback.
	 * @param conn
	 * @param e
	 * @throws ExecuteSQLException
	 */
	public static void connRollback(Connection conn,Exception e) 
			throws ExecuteSQLException{
		try {
			if ( conn.getAutoCommit() ) {
				return;
			}
			if ( null != conn && !conn.isClosed() )
				try {
					conn.rollback();
				} catch (SQLException e1) {
					throw new ExecuteSQLException(
							"Executing sql is failure.",e1.fillInStackTrace());
				}
		} catch (SQLException e1) {
			throw new ExecuteSQLException(
					"Executing sql is failure.",e1.fillInStackTrace());
		}
	}
	
	/**
	 * Release.
	 * @param conn
	 * @param rs
	 * @param stmt
	 * @throws ExecuteSQLException
	 */
	public static void connRelease(Connection conn,ResultSet rs,Statement stmt) 
			throws ExecuteSQLException{
		if ( conn != null ) {
			try {
				if ( rs != null )
					rs.close();
				rs = null;
				if ( stmt != null )
					stmt.close();
				stmt = null;
				conn.close();
				conn = null;
			} catch (SQLException e1) {
				throw new ExecuteSQLException("Executing sql is failure.",e1.fillInStackTrace());
			}
		}
	}
		

	/**
	 * Get fields of Object
	 * @param obj Object
	 * @return Field[]
	 * */
	public static Field[] getFields(Object obj) throws ExecuteSQLException{
		if ( CommonTools.isNull(obj) ) {
			throw new ExecuteSQLException("Object is null.");
		}
		Class<?> c = obj.getClass();
		java.lang.reflect.Field[] fields = c.getDeclaredFields();
		if ( fields == null )
			throw new ExecuteSQLException("Fields is null.");
		return fields;
	}
	
	/**
	 * Execute sql
	 * @param st PreparedStatement Object
	 * */
	public static ResultSet query(String sql, Connection conn) throws ExecuteSQLException {
		if ( CommonTools.isNull(sql) ) {
			throw new ExecuteSQLException("Sql is null.");
		}
		if ( CommonTools.isNull(conn) ) {
			throw new ExecuteSQLException("Connection is null.");
		}
		try {
			ResultSet rs = conn.createStatement().executeQuery(sql);
			return rs;
		} catch (SQLException e) {
			throw new ExecuteSQLException("Executing PreparedStatement is failure.",e.fillInStackTrace());
		}
	}
	
	/**
	 * Transform ResultSet to Object.
	 * @param rs ResultSet object
	 * @param c Class of JavaBean
	 * @return JavaBean object
	 * */
	public static Object parseResultSetToObject(ResultSet rs,
			Class<?> clazz,List<OutputMapping> mapping) 
			throws Exception{
		if ( null == rs )
			throw new Exception("ResultSet is null.");
		if ( null == clazz )
			throw new Exception("Class<?> is null.");
		Object obj = clazz.newInstance();
		java.lang.reflect.Field[] fields = clazz.getDeclaredFields();
		if ( fields == null )
			throw new Exception("Fields is null.");
		
		String name = null;//field name
		Object value = null;//field value
		Class<?> type = null;
		String rsKey = null;
		for ( OutputMapping m : mapping ) {
			type = str2Type(m.getTargetClass());
			rsKey = m.getSourceName();
			name = m.getTargetName();
			value = rs2Obj(type, rs, rsKey);
			setFieldValue(name, type, obj, value);
		}
		return obj;
	}
	
	/**
	 * Set field value by set method of object
	 * @param fieldName field name
	 * @param fieldType field type
	 * @param obj owner of field
	 * @param parameter value of filed
	 * @throws OperationsException
	 * */
	public static void setFieldValue(
			String fieldName,Class<?> fieldType,Object obj,Object parameter) 
	throws Exception{
		Class<?> clazz = obj.getClass();
		Method m = null;
		if ( null != parameter ) {
			StringBuilder sb = new StringBuilder();
			if ( fieldName.matches("is[A-Z]+[\\w|\\W]*") ) {
				fieldName = fieldName.replace("is", "");
				sb.append("set").
					append(fieldName.substring(0,1).toUpperCase()).
					append(fieldName.substring(1, fieldName.length()));
			} else
				sb.append("set").
					append(fieldName.substring(0,1).toUpperCase()).
					append(fieldName.substring(1, fieldName.length()));
			m = clazz.getMethod(sb.toString(),fieldType);
			//Transform string to int etc.
			if ( String.class == parameter.getClass() && String.class != fieldType ) {
				String v = parameter.toString().trim();
				if ( boolean.class == fieldType || Boolean.class == fieldType ) {
					parameter = Boolean.parseBoolean(v);
				} else if ( int.class == fieldType || Integer.class == fieldType ) {
					parameter = Integer.valueOf(v).intValue();
				} else if ( long.class == fieldType || Long.class == fieldType ) {
					parameter = Long.valueOf(v).longValue();
				} else if ( float.class == fieldType || Float.class == fieldType ) {
					parameter = Float.valueOf(v).floatValue();
				} else if ( double.class == fieldType || Double.class == fieldType ) {
					parameter = Double.valueOf(v).doubleValue();
				} else if ( short.class == fieldType || Short.class == fieldType ) {
					parameter = Short.valueOf(v).shortValue();
				} else if ( Byte.class == fieldType || byte.class == fieldType ) {
					parameter = Byte.valueOf(v).byteValue();
				} else if ( Character.class == fieldType || char.class == fieldType ) {
					parameter = String.valueOf(v);
				} 
			}
			m.invoke(obj, new Object[]{parameter});		
		}	
	}
	
	/**
	 * Determines whether the incoming object is a int number.
	 * @param obj
	 * @return
	 */
	public final static boolean isInteger(Object obj) {
		if ( obj == null )
			return false;
		if ( obj.toString().matches("(\\-|\\+)?\\d+") ) {
			return true;
		}
		return false;
	}
	
	/**
	 * Determines whether the incoming object is a double number.
	 * @param obj
	 * @return
	 */
	public final static boolean isDouble(Object obj) {
		if ( obj == null )
			return false;
		if ( obj.toString().matches("(\\-|\\+)?\\d+(\\.\\d+)?") ) {
			return true;
		}
		return false;
	}
	
	/**
	 * Get generate keys after insert.
	 * @param obj
	 * @param st
	 * @throws ExecuteSQLException
	 */
	public static void getGeneratedKeys(Object obj, Statement st) throws ExecuteSQLException {
		if ( CommonTools.isNull(obj) ) {
			throw new ExecuteSQLException("Object is null.");
		}
		if ( CommonTools.isNull(st) ) {
			throw new ExecuteSQLException("PreparedStatement is null.");
		}
		try {
			String strId = "id";
			Class<?> clazz = obj.getClass();
			Field[] fields = clazz.getDeclaredFields();
			Class<?> fClazz = null;
			boolean exist = false;
			for ( Field f : fields ) {
				if ( f.getName().equals(strId) ) {
					fClazz = f.getType();
					exist = true; 
					break;
				}
			}
			if ( exist ) {
				ResultSet rs = st.getGeneratedKeys();
				if ( rs.next() ) {
					int id = rs.getInt(1);
					DefaultBaseAction.setFieldValue(strId, fClazz, obj, id);
				}
			}
		} catch (Exception e) {
			Logger.getInstance().error("There is an error on generating id of data.", e);
		}
		
	}
}
