/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.basic;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Modifier;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.sql.Blob;
import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.List;
import java.util.Vector;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.config.CommonTools;
import com.nonesole.persistence2.config.bean.Action;
import com.nonesole.persistence2.config.bean.OSMObject;
import com.nonesole.persistence2.sql.ConfigLoader;
import com.nonesole.persistence2.sql.basic.input.InputTransform;
import com.nonesole.persistence2.sql.basic.output.OutputTransform;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;



/**
 * This class implements IBasicAction interface for executing PreparedStatement.
 * @author jack lee
 * @version 1.0
 * @date 2016-10-29
 */
public class DefaultBasicPreparedAction implements IBasicAction{
    
	private Logger logger;
	private static final DefaultBasicPreparedAction action;
	static {
		action = new DefaultBasicPreparedAction();
	}
	private DefaultBasicPreparedAction(){
		logger = Logger.getInstance();
	}
	
	/**
	 * @return
	 */
	public static DefaultBasicPreparedAction getInstance(){
		return action;
	}

	@Override
	public void executeAction(Action action, Connection conn,
			boolean partOfTransaction, InputTransform inputTransform)
			throws Exception {
		if ( CommonTools.isNull(action) ) {
			throw new ExecuteSQLException("Action object is null.");
		}
		if ( CommonTools.isNull(conn) ) {
			throw new ExecuteSQLException("Connection is null.");
		}
		if ( CommonTools.isNull(inputTransform) ) {
			throw new ExecuteSQLException("InputTransform object is null.");
		}
		
		inputTransform.transform(action);
		List<String> sqls = inputTransform.getSQLList();
		PreparedStatement pstmt = null;
		DefaultBaseAction.commitConnection(conn, false);
		for ( String sql : sqls ) {
			if ( inputTransform.getValueMap(sql) == null ) {
				DefaultBaseAction.executeNotCommit(sql, conn);
			} else {
				pstmt = getPreparedStatement(conn, sql);
				setValues(
						pstmt, 
						inputTransform.getValueMap(sql), 
						inputTransform.getFieldTypeMap(sql), 
						inputTransform.getBlobPositionMap(sql), 0);
				execute(pstmt);	
			}
		}
		
		if ( !partOfTransaction ) {
			DefaultBaseAction.commitConnection(conn, true);
			DefaultBaseAction.connRelease(conn, null, pstmt);
		}
	}

	@Override
	public void queryAction(Action action, Connection conn,
			InputTransform inputTransform, OutputTransform<?> outTransform)
			throws Exception {
		if ( CommonTools.isNull(action) ) {
			throw new ExecuteSQLException("Action object is null.");
		}
		if ( CommonTools.isNull(conn) ) {
			throw new ExecuteSQLException("Connection is null.");
		}
		if ( CommonTools.isNull(inputTransform) ) {
			throw new ExecuteSQLException("InputTransform object is null.");
		}
		if ( CommonTools.isNull(outTransform) ) {
			throw new ExecuteSQLException("OutputTransform object is null.");
		}
		
		inputTransform.transform(action);
		List<String> sqls = inputTransform.getSQLList();
		PreparedStatement pstmt = null;
		DefaultBaseAction.commitConnection(conn, false);
		String sql = null;
		int lastIndex = sqls.size() - 1;
		ResultSet rs = null;
		for ( int i = 0 ; i < sqls.size() ; i++ ) {
			sql = sqls.get(i);
			logger.debug("DefaultBasicPreparedAction:sql="+sql, null);
			if ( inputTransform.getValueMap(sql) == null ) {
				//�޲���������ֱ��ִ��SQL
				rs = DefaultBaseAction.query(sql, conn);
				outTransform.transform(rs);
			} else {
				pstmt = getPreparedStatement(conn, sql);
				setValues(
						pstmt, 
						inputTransform.getValueMap(sql), 
						inputTransform.getFieldTypeMap(sql), 
						inputTransform.getBlobPositionMap(sql), 0);
				//ִ��SQL
				if ( i < lastIndex ) {
					execute(pstmt);	
				} else {
					rs = query(pstmt);
					outTransform.transform(rs);
				}
			}
		}
		DefaultBaseAction.commitConnection(conn, true);
		DefaultBaseAction.connRelease(conn, rs, pstmt);
	}
			
	/**
	 * Set values to prepared statement to finish integral sql.
	 * Just blob not clob.
	 * @param st PreparedStatement which has prepared sql
	 * @param values to '?' in sql
	 * @param type of values
	 * @param blob's position in values 
	 * @param startPosition start position of values 
	 * */
	public static void setValues(
			PreparedStatement st,
			List<Object> values,
			List<Class<?>> fTypes,
			List<Integer> blobPosition,
			int startPosition)
	throws ExecuteSQLException {
		if ( CommonTools.isNull(st) ) {
			throw new ExecuteSQLException("PreparedStatement object is null.");
		}
		if ( CommonTools.isNull(values) ) {
			throw new ExecuteSQLException("Values are null.");
		}
		if ( CommonTools.isNull(fTypes) ) {
			throw new ExecuteSQLException("Class type is null.");
		}
		try {
			Class<?> type = null;
			int i = 0;
			int position = (startPosition<=0)?1:startPosition;//Get start position
			for ( Object obj : values ) {
				type = fTypes.get(i);//Get type
				if ( boolean.class == type || Boolean.class == type ) {
					obj = (obj==null)?false:obj;
					if ( obj.getClass() == String.class ) {
						st.setBoolean(position, Boolean.valueOf((String) obj));
					} else {
						st.setBoolean(position, (Boolean)obj);
					}
				} else if ( int.class == type || Integer.class == type ) {
					obj = (obj==null)?0:obj;
					if ( obj.getClass() == String.class ) {
						st.setInt(position, Integer.valueOf((String) obj));
					} else {
						st.setInt(position, (Integer)obj);
					}
				} else if ( long.class == type || Long.class == type ) {
					obj = (obj==null)?0:obj;
					if ( obj.getClass() == String.class ) {
						st.setLong(position, Long.valueOf((String) obj));
					} else {
						st.setLong(position, (Long)obj);
					}
				} else if ( float.class == type || Float.class == type ) {
					obj = (obj==null)?0.0f:obj;
					if ( obj.getClass() == String.class ) {
						st.setFloat(position, Float.valueOf((String) obj));
					} else {
						st.setFloat(position, (Float)obj);
					}
				} else if ( double.class == type || Double.class == type ) {
					obj = (obj==null)?0.0d:obj;
					if ( obj.getClass() == String.class ) {
						st.setDouble(position, Double.valueOf((String) obj));
					} else {
						st.setDouble(position, (Double)obj);
					}
				} else if ( short.class == type || Short.class == type ) {
					obj = (obj==null)?0:obj;
					if ( obj.getClass() == String.class ) {
						st.setShort(position, Short.valueOf((String) obj));
					} else {
						st.setShort(position, (Short)obj);
					}
				} else if ( Byte.class == type || byte.class == type ) {
					obj = (obj==null)?new Byte(""):obj;
					if ( obj.getClass() == String.class ) {
						st.setByte(position, Byte.valueOf((String) obj));
					} else {
						st.setByte(position, (Byte)obj);
					}
				} else if ( String.class == type ) {
					obj = (obj==null)?"":obj;
					st.setString(position, (String)obj);
				} else if ( type == Date.class  ){
					obj = (obj==null)?new Date(System.currentTimeMillis()):obj;
					st.setDate(position, (Date)obj);
				} else if ( type == Time.class  ){
					obj = (obj==null)?new Time(System.currentTimeMillis()):obj;
					st.setTime(position, (Time)obj);
				} else if ( type == Timestamp.class  ){
					obj = (obj==null)?new Timestamp(System.currentTimeMillis()):obj;
					st.setTimestamp(position, (Timestamp)obj);
				} else if ( type == Character.class || type == char.class ){
					obj = (obj==null)?"":obj;
					st.setString(position, String.valueOf(obj));
				} else if ( type == BigDecimal.class  ){
					obj = (obj==null)?0:obj;
					st.setBigDecimal(position, (BigDecimal)obj);
				} else if ( type == BigInteger.class ){
					obj = (obj==null)?0:obj;
					st.setLong(position, ((BigInteger)obj).longValue());
				} else if ( type == byte[].class ){
					setBlob(st,blobPosition,position);//deal with blob
				} else {
					obj = (obj==null)?type.newInstance():obj;
					st.setObject(position, obj);
				} 
				i++;
				position++;
			}
		} catch (Exception e) {
			throw new ExecuteSQLException(
					"Setting value for preparement is failure.",
					e.fillInStackTrace());
		} 
	}
	
	/**
	 * Deal with blob value
	 * @param st PreparedStatement object
	 * @param blob's position in values
	 * @param fieldPosition field position in fields
	 * */
	public static void setBlob(
			PreparedStatement st,
			List<Integer> blobPosition,
			int fieldPosition) 
	throws ExecuteSQLException {	
		if ( CommonTools.isNull(st) ) {
			throw new ExecuteSQLException("PreparedStatement object is null.");
		}
		if ( CommonTools.isNull(blobPosition) ) {
			throw new ExecuteSQLException("Blob position is null.");
		}
		//Blob <-> byte[],This frame do not deal with blob in other types
		try {
			st.setBinaryStream(fieldPosition, 
					new DataInputStream(new ByteArrayInputStream("".getBytes())) ,
					0);
		} catch (SQLException e) {
			throw new ExecuteSQLException(
					"Found mistakes when set Blob field.",
					e.fillInStackTrace());
		}
		blobPosition.add(fieldPosition);
	}
	
	/**
	 * Deal with blob field
	 * @param conn Connection Object
	 * @param obj Object needs save
	 * @param blobPosition blob's position in fields
	 * @param v list of values 
	 * @throws InvocationTargetException 
	 * @throws IllegalArgumentException 
	 * @throws IllegalAccessException 
	 * @throws SecurityException 
	 * @throws NoSuchMethodException 
	 * */
	public static void saveBlob(Connection conn,
			Object obj,
			Vector<Integer> blobPosition,
			Vector<Object> v) 
	throws ExecuteSQLException, NoSuchMethodException, SecurityException, IllegalAccessException, IllegalArgumentException, InvocationTargetException{
		if ( CommonTools.isNull(obj) ) {
			throw new ExecuteSQLException("Object is null.");
		}
		if ( CommonTools.isNull(blobPosition) ) {
			throw new ExecuteSQLException("Blob position is null.");
		}
		if ( CommonTools.isNull(v) ) {
			throw new ExecuteSQLException("Values are null.");
		}
		
		if ( blobPosition.size() < 1 || v.size() < 1 ) {
			return;
		}
		
		ResultSet rs = null;
		Statement stt = null;
		try {
			stt = conn.createStatement();
			rs = stt.executeQuery(getQuerySQLWithPrimeryKey(obj));
			if (rs.next()) {
				Blob blob;
				int size;
				for ( Integer i : blobPosition ) {
					size = ((byte[])v.get(i-1)).length;
					blob = rs.getBlob(i);
					blob.setBytes(1, (byte[])v.get(i-1), 0, size);
				}				
			}
			DefaultBaseAction.connRelease(null, rs, stt);
		} catch (SQLException e) {
			DefaultBaseAction.connRelease(conn, rs, stt);
			throw new ExecuteSQLException(
					"Executing PreparedStatement is failure.",
					e.fillInStackTrace());
		}
	}
	
	/**
	 * Get query sql whose id as condition.
	 * @param conn Connection Object
	 * @param obj object for query
	 * @return SQL
	 * @throws InvocationTargetException 
	 * @throws IllegalArgumentException 
	 * @throws IllegalAccessException 
	 * @throws SecurityException 
	 * @throws NoSuchMethodException 
	 * */
	public static String getQuerySQLWithPrimeryKey(Object obj) 
	throws ExecuteSQLException, NoSuchMethodException, SecurityException, IllegalAccessException, IllegalArgumentException, InvocationTargetException{
		if ( CommonTools.isNull(obj) ) {
			throw new ExecuteSQLException("Object is null.");
		}
		OSMObject osm = getOSM(obj);
		if ( CommonTools.isNull(osm) ) {
			throw new ExecuteSQLException("OSM is null.");
		}

		Field[] fields = DefaultBaseAction.getFields(obj);
		String name;//field name
		Class<?> type = null;//field type
		StringBuilder set = new StringBuilder();//sql like 'field=value'
		StringBuilder where = new StringBuilder(); //where sql
		StringBuilder temp = null;
		Object value;//value of string type
		
		String[] ks = osm.getPrimaryKeys();
		Vector<String> key = DefaultBaseAction.arrayToVector(ks);
		for( Field field : fields ){
			name = field.getName();
			type = field.getType();	
			int modifier = field.getModifiers();
			if ( Modifier.isFinal(modifier) ||
					(Modifier.isStatic(modifier) && Modifier.isPublic(modifier) ) ) {
				//public + static,no need set and get method
				continue;
			} 				
			if ( key.contains(name) ) {
				temp = new StringBuilder();
				//get value
				value = DefaultBaseAction.invoke(field, obj);
				temp.append(name).append("=");
				//whether or not use '' around character 
				DefaultBaseAction.handleCharacter(type,temp,value);
				//join set sub sentence and where sub setnece 
				set.append(temp).append(",");
				where.append(temp).append(" and ");
			}
		} 
		//delete last ','
		int point = set.lastIndexOf(",");
		set.delete(point, point+1);
		point = where.lastIndexOf(" and ");
		where.delete(point, point+6);
		
		StringBuilder sql = new StringBuilder();
		sql.append("select * from")
			.append(" ")
			.append(osm.getTableName())
			.append(" ")
			.append("where")
			.append(" ")
			.append(where);
		
		return sql.toString();
	}
	
	/**
	 * �ӻ�����ȡ��ָ��Object��Ӧ��ORM����
	 * @param obj
	 * @return ObjectSqlMapping
	 * @throws ExecuteSQLException
	 */
	public static OSMObject getOSM(Object obj) throws ExecuteSQLException {
		if ( CommonTools.isNull(obj) ) {
			throw new ExecuteSQLException("Object is null.");
		}
		return ConfigLoader.getInstance().getObjectSqlMapping(obj.getClass().getName());
	}
	
	
	/**
	 * Get prepared statement by conn and sql
	 * @param conn Connection
	 * @return Statement
	 * */
	public static PreparedStatement getPreparedStatement(Connection conn,String sql) 
	throws ExecuteSQLException{
		if ( CommonTools.isNull(conn) ) {
			throw new ExecuteSQLException("Connection is null.");
		}
		if ( CommonTools.isNull(sql) ) {
			throw new ExecuteSQLException("SQL is null.");
		}
		try {
			return conn.prepareStatement(sql);			
		} catch (SQLException e) {
			throw new ExecuteSQLException("Creating statement is failure.",e.fillInStackTrace());
		}
	}
	
	/**
	 * Execute sql
	 * @param st PreparedStatement Object
	 * */
	public static void execute(PreparedStatement st) throws ExecuteSQLException {
		if ( CommonTools.isNull(st) ) {
			throw new ExecuteSQLException("PreparedStatement is null.");
		}
		Connection conn = null;
		try {
			//Logger.getInstance().debug("PreparedStatement:SQL" + st.toString(), null);
			Logger.getInstance().info("PreparedStatement. " + st.toString(), null);
			conn = st.getConnection();
			st.execute();
			st.close();
			st = null;
		} catch (SQLException e) {
			DefaultBaseAction.connRelease(conn, null, null);
			throw new ExecuteSQLException("Executing PreparedStatement is failure.",e.fillInStackTrace());
		}
	}
	
	/**
	 * Execute sql
	 * @param st PreparedStatement Object
	 * */
	public static void executeSave(PreparedStatement st, Object obj) throws ExecuteSQLException {
		if ( CommonTools.isNull(st) ) {
			throw new ExecuteSQLException("PreparedStatement is null.");
		}
		Connection conn = null;
		try {
			//Logger.getInstance().debug("PreparedStatement:SQL" + st.toString(), null);
			Logger.getInstance().info("PreparedStatement. " + st.toString(), null);
			conn = st.getConnection();
			st.execute();
			DefaultBaseAction.getGeneratedKeys(obj, st);
			st.close();
			st = null;
		} catch (SQLException e) {
			DefaultBaseAction.connRelease(conn, null, null);
			throw new ExecuteSQLException("Executing PreparedStatement is failure.",e.fillInStackTrace());
		}
	}
	
	/**
	 * Execute sql
	 * @param st PreparedStatement Object
	 * */
	public static ResultSet query(PreparedStatement st) throws ExecuteSQLException {
		if ( CommonTools.isNull(st) ) {
			throw new ExecuteSQLException("PreparedStatement is null.");
		}
		try {
			Logger.getInstance().info("PreparedStatement. " + st.toString(), null);
			ResultSet rs = st.executeQuery();
			return rs;
		} catch (SQLException e) {
			throw new ExecuteSQLException("Executing PreparedStatement is failure.",e.fillInStackTrace());
		}
	}
	
}
