/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.basic;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.List;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.config.CommonTools;
import com.nonesole.persistence2.config.bean.Action;
import com.nonesole.persistence2.sql.basic.input.InputTransform;
import com.nonesole.persistence2.sql.basic.output.OutputTransform;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;

/**
 * This class implements IBasicAction interface for executing Statement.
 * @author jack lee
 * @version 1.0
 * @date 2016-10-30
 */
public class DefaultBasicStatementAction implements IBasicAction{

	private Logger logger;
	private static final DefaultBasicStatementAction action;
	static {
		action = new DefaultBasicStatementAction();
	}
	private DefaultBasicStatementAction(){
		logger = Logger.getInstance();
	}
	
	/**
	 * @return
	 */
	public static DefaultBasicStatementAction getInstance(){
		return action;
	}
	
	@Override
	public void executeAction(Action action, Connection conn,
			boolean partOfTransaction, InputTransform inputTransform)
			throws Exception {
		if ( CommonTools.isNull(action) ) {
			throw new ExecuteSQLException("Action object is null.");
		}
		if ( CommonTools.isNull(conn) ) {
			throw new ExecuteSQLException("Connection is null.");
		}
		if ( CommonTools.isNull(inputTransform) ) {
			throw new ExecuteSQLException("InputTransform object is null.");
		}
		
		inputTransform.transform(action);
		List<String> sqls = inputTransform.getSQLList();
		Statement stmt = conn.createStatement();
		DefaultBaseAction.commitConnection(conn, false);
		for ( String sql : sqls ) {//ִ��SQL
			stmt.addBatch(sql);
			logger.debug("DefaultBasicStatementAction:sql="+sql, null);
		}
		Logger.getInstance().info("Statement. " + stmt.toString(), null);
		stmt.executeBatch();
		
		if ( !partOfTransaction ) {
			DefaultBaseAction.commitConnection(conn, true);
			DefaultBaseAction.connRelease(conn, null, stmt);
		}
	}

	@Override
	public void queryAction(Action action, Connection conn,
			InputTransform inputTransform, OutputTransform<?> outTransform)
			throws Exception {
		if ( CommonTools.isNull(action) ) {
			throw new ExecuteSQLException("Action object is null.");
		}
		if ( CommonTools.isNull(conn) ) {
			throw new ExecuteSQLException("Connection is null.");
		}
		if ( CommonTools.isNull(inputTransform) ) {
			throw new ExecuteSQLException("InputTransform object is null.");
		}
		if ( CommonTools.isNull(outTransform) ) {
			throw new ExecuteSQLException("OutputTransform object is null.");
		}
		
		inputTransform.transform(action);
		List<String> sqls = inputTransform.getSQLList();
		Statement stmt = conn.createStatement();
		DefaultBaseAction.commitConnection(conn, false);
		int lastIndex = sqls.size() - 1;
		ResultSet rs = null;
		for ( int i = 0 ; i < lastIndex ; i++ ) {//ִ��SQL
			stmt.addBatch(sqls.get(i));
		}
		Logger.getInstance().info("Statement. " + stmt.toString(), null);
		stmt.executeBatch();
		
		rs = stmt.executeQuery(sqls.get(lastIndex));
		outTransform.transform(rs);
		
		DefaultBaseAction.commitConnection(conn, true);
		DefaultBaseAction.connRelease(conn, rs, stmt);
	}
	
}
