/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.custom;

import java.sql.Connection;
import java.util.List;
import java.util.Map;

import com.nonesole.persistence2.config.bean.Action;
import com.nonesole.persistence2.connection.exception.OperationsException;
import com.nonesole.persistence2.connection.pool.MultiConnectionPool;
import com.nonesole.persistence2.sql.ConfigLoader;
import com.nonesole.persistence2.sql.basic.DefaultBaseAction;
import com.nonesole.persistence2.sql.basic.DefaultBasicPreparedAction;
import com.nonesole.persistence2.sql.basic.DefaultBasicStatementAction;
import com.nonesole.persistence2.sql.basic.input.MapInputTransform;
import com.nonesole.persistence2.sql.basic.output.SimpleDyadicListOutputTransform;
import com.nonesole.persistence2.sql.define.IDyadicArrayValueAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;

/** 
 * Implementation class of IDyadicArrayValueAction
 * @author jack lee
 * @version 1.0 - 2016-06-26
 */
public class DefaultDyadicArrayValueAction extends DefaultBaseAction implements IDyadicArrayValueAction {

	private String defaultDatasource = null;

	private MultiConnectionPool pool = null;
	
	private DefaultBasicPreparedAction preparedAction = null;
	
	private DefaultBasicStatementAction statementAction = null;
		
	/**
	 * Constructor
	 */
	public DefaultDyadicArrayValueAction(){
		this(true);
	}
	
	/**
	 * Constructor
	 * @param isPreparedStatement
	 */
	public DefaultDyadicArrayValueAction(boolean isPreparedStatement){
		this(null,isPreparedStatement);
	}
	
	/**
	 * Constructor
	 * @param datasrouceName
	 * @param isPreparedStatement
	 */
	public DefaultDyadicArrayValueAction(String datasrouceName){
		this(datasrouceName,true);
	}
	
	/**
	 * Constructor
	 * @param datasrouceName
	 * @param isPreparedStatement
	 */
	public DefaultDyadicArrayValueAction(String datasrouceName,boolean isPreparedStatement){
		super(isPreparedStatement);
		this.defaultDatasource = datasrouceName;
		pool = MultiConnectionPool.getInstance();
		preparedAction = DefaultBasicPreparedAction.getInstance();
		statementAction = DefaultBasicStatementAction.getInstance();
	}
	
	/**
	 * The method doAction should be used by all public methods 
	 * of this class. The return value is a dyadic list of 
	 * single-value.
	 * 
	 * @param actionName
	 * @param input
	 * @param conn
	 * @return
	 * @throws Exception
	 */
	protected List<List<Object>> doAction(String actionName, Map<String,Object> input, Connection conn, boolean isPrepared) 
	throws Exception{
		Action action = ConfigLoader.getInstance().getAction(actionName);
		MapInputTransform inputTransform = new MapInputTransform(input, isPrepared);
		SimpleDyadicListOutputTransform outTransform = new SimpleDyadicListOutputTransform(action);
		if ( isPrepared ) {	
			preparedAction.queryAction(action, conn, inputTransform, outTransform);
		} else {
			statementAction.queryAction(action, conn, inputTransform, outTransform);
		}	
		return outTransform.getObject();
	}
	
	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicArrayValueAction#shortDyadicArrayAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public Short[][] shortDyadicArrayAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.shortDyadicArrayAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicArrayValueAction#shortDyadicArrayAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public Short[][] shortDyadicArrayAction(String actionName, Map<String,Object> input,
			Connection conn) throws ExecuteSQLException {
		return this.shortDyadicArrayAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public Short[][] shortDyadicArrayAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		List<List<Object>> objects = null;
		try {
			objects = this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
		if ( objects != null && !objects.isEmpty() ) {
			Short[][] r = new Short[objects.size()][objects.get(0).size()];
			for ( int i = 0 ; i < objects.size() ; i++ ) {
				objects.get(i).toArray(r[i]);
			}
			return r;
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicArrayValueAction#intDyadicArrayAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public Integer[][] intDyadicArrayAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.intDyadicArrayAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicArrayValueAction#intDyadicArrayAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public Integer[][] intDyadicArrayAction(String actionName, Map<String,Object> input,
			Connection conn) throws ExecuteSQLException {
		return this.intDyadicArrayAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public Integer[][] intDyadicArrayAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		List<List<Object>> objects = null;
		try {
			objects = this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
		if ( objects != null && !objects.isEmpty() ) {
			Integer[][] r = new Integer[objects.size()][objects.get(0).size()];
			for ( int i = 0 ; i < objects.size() ; i++ ) {
				objects.get(i).toArray(r[i]);
			}
			return r;
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicArrayValueAction#longDyadicArrayAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public Long[][] longDyadicArrayAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.longDyadicArrayAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicArrayValueAction#longDyadicArrayAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public Long[][] longDyadicArrayAction(String actionName, Map<String,Object> input,
			Connection conn) throws ExecuteSQLException {
		return this.longDyadicArrayAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public Long[][] longDyadicArrayAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		List<List<Object>> objects = null;
		try {
			objects = this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
		if ( objects != null && !objects.isEmpty() ) {
			Long[][] r = new Long[objects.size()][objects.get(0).size()];
			for ( int i = 0 ; i < objects.size() ; i++ ) {
				objects.get(i).toArray(r[i]);
			}
			return r;
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicArrayValueAction#floatDyadicArrayAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public Float[][] floatDyadicArrayAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.floatDyadicArrayAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicArrayValueAction#floatDyadicArrayAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public Float[][] floatDyadicArrayAction(String actionName, Map<String,Object> input,
			Connection conn) throws ExecuteSQLException {
		return this.floatDyadicArrayAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public Float[][] floatDyadicArrayAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		List<List<Object>> objects = null;
		try {
			objects = this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
		if ( objects != null && !objects.isEmpty() ) {
			Float[][] r = new Float[objects.size()][objects.get(0).size()];
			for ( int i = 0 ; i < objects.size() ; i++ ) {
				objects.get(i).toArray(r[i]);
			}
			return r;
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicArrayValueAction#doubleDyadicArrayAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public Double[][] doubleDyadicArrayAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.doubleDyadicArrayAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicArrayValueAction#doubleDyadicArrayAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public Double[][] doubleDyadicArrayAction(String actionName,
			Map<String,Object> input, Connection conn) throws ExecuteSQLException {
		return this.doubleDyadicArrayAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public Double[][] doubleDyadicArrayAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		List<List<Object>> objects = null;
		try {
			objects = this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
		if ( objects != null && !objects.isEmpty() ) {
			Double[][] r = new Double[objects.size()][objects.get(0).size()];
			for ( int i = 0 ; i < objects.size() ; i++ ) {
				objects.get(i).toArray(r[i]);
			}
			return r;
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicArrayValueAction#booleanDyadicArrayAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public Boolean[][] booleanDyadicArrayAction(String actionName,
			Map<String,Object> input) throws ExecuteSQLException {
		try {
			return this.booleanDyadicArrayAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicArrayValueAction#booleanDyadicArrayAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public Boolean[][] booleanDyadicArrayAction(String actionName,
			Map<String,Object> input, Connection conn) throws ExecuteSQLException {
		return this.booleanDyadicArrayAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public Boolean[][] booleanDyadicArrayAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		List<List<Object>> objects = null;
		try {
			objects = this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
		if ( objects != null && !objects.isEmpty() ) {
			Boolean[][] r = new Boolean[objects.size()][objects.get(0).size()];
			for ( int i = 0 ; i < objects.size() ; i++ ) {
				objects.get(i).toArray(r[i]);
			}
			return r;
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicArrayValueAction#stringDyadicArrayAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public String[][] stringDyadicArrayAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.stringDyadicArrayAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicArrayValueAction#stringDyadicArrayAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public String[][] stringDyadicArrayAction(String actionName,
			Map<String,Object> input, Connection conn) throws ExecuteSQLException {
		return this.stringDyadicArrayAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public String[][] stringDyadicArrayAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		List<List<Object>> objects = null;
		try {
			objects = this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
		if ( objects != null && !objects.isEmpty() ) {
			String[][] r = new String[objects.size()][objects.get(0).size()];
			for ( int i = 0 ; i < objects.size() ; i++ ) {
				objects.get(i).toArray(r[i]);
			}
			return r;
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicArrayValueAction#objDyadicArrayAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public Object[][] objDyadicArrayAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.objDyadicArrayAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicArrayValueAction#objDyadicArrayAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public Object[][] objDyadicArrayAction(String actionName, Map<String,Object> input,
			Connection conn) throws ExecuteSQLException {
		return this.objDyadicArrayAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public Object[][] objDyadicArrayAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		List<List<Object>> objects = null;
		try {
			objects = this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
		if ( objects != null && !objects.isEmpty() ) {
			Object[][] r = new Object[objects.size()][objects.get(0).size()];
			for ( int i = 0 ; i < objects.size() ; i++ ) {
				objects.get(i).toArray(r[i]);
			}
			return r;
		}
		return null;
	}

}
