/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.custom;

import java.sql.Connection;
import java.util.List;
import java.util.Map;

import com.nonesole.persistence2.config.bean.Action;
import com.nonesole.persistence2.connection.exception.OperationsException;
import com.nonesole.persistence2.connection.pool.MultiConnectionPool;
import com.nonesole.persistence2.sql.ConfigLoader;
import com.nonesole.persistence2.sql.basic.DefaultBaseAction;
import com.nonesole.persistence2.sql.basic.DefaultBasicPreparedAction;
import com.nonesole.persistence2.sql.basic.DefaultBasicStatementAction;
import com.nonesole.persistence2.sql.basic.input.MapInputTransform;
import com.nonesole.persistence2.sql.basic.output.ListMaker;
import com.nonesole.persistence2.sql.basic.output.SimpleDyadicListOutputTransform;
import com.nonesole.persistence2.sql.define.IDyadicListValueAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;

/**
 * Implementation class of IDyadicListValueAction
 * @author jack lee
 * @version 1.0 - 2016-06-26
 */
public class DefaultDyadicListValueAction extends DefaultBaseAction implements IDyadicListValueAction {

	private String defaultDatasource = null;

	private MultiConnectionPool pool = null;
	
	private DefaultBasicPreparedAction preparedAction = null;
	
	private DefaultBasicStatementAction statementAction = null;
	
	/**
	 * Constructor
	 */
	public DefaultDyadicListValueAction(){
		this(true);
	}
	
	/**
	 * Constructor
	 * @param isPreparedStatement
	 */
	public DefaultDyadicListValueAction(boolean isPreparedStatement){
		this(null,isPreparedStatement);
	}
	
	/**
	 * Constructor
	 * @param datasrouceName
	 * @param isPreparedStatement
	 */
	public DefaultDyadicListValueAction(String datasrouceName){
		this(datasrouceName,true);
	}
	
	/**
	 * Constructor
	 * @param datasrouceName
	 * @param isPreparedStatement
	 */
	public DefaultDyadicListValueAction(String datasrouceName,boolean isPreparedStatement){
		super(isPreparedStatement);
		this.defaultDatasource = datasrouceName;
		pool = MultiConnectionPool.getInstance();
		preparedAction = DefaultBasicPreparedAction.getInstance();
		statementAction = DefaultBasicStatementAction.getInstance();
	}
	
	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicListValueAction#shortDyadicListAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public List<List<Short>> shortDyadicListAction(String actionName,
			Map<String,Object> input) throws ExecuteSQLException {
		try {
			return this.shortDyadicListAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/**
	 * The method doAction should be used by all public methods 
	 * of this class. The return value is a dyadic list of 
	 * single-value.
	 * 
	 * @param actionName
	 * @param input
	 * @param conn
	 * @param isPrepared
	 * @return
	 * @throws Exception
	 */
	protected List<List<Object>> doAction(String actionName, Map<String,Object> input, Connection conn, boolean isPrepared) 
	throws Exception{
		Action action = ConfigLoader.getInstance().getAction(actionName);
		MapInputTransform inputTransform = new MapInputTransform(input, isPrepared);
		SimpleDyadicListOutputTransform outTransform = new SimpleDyadicListOutputTransform(action);
		if ( this.isPreparedStatement ) {	
			preparedAction.queryAction(action, conn, inputTransform, outTransform);
		} else {
			statementAction.queryAction(action, conn, inputTransform, outTransform);
		}	
		return outTransform.getObject();
	}
	
	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicListValueAction#shortDyadicListAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public List<List<Short>> shortDyadicListAction(String actionName,
			Map<String,Object> input, Connection conn) throws ExecuteSQLException {
		return this.shortDyadicListAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public List<List<Short>> shortDyadicListAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		List<List<Object>> objects = null;
		try {
			objects = this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
		return new ListMaker<Short>().transform(objects);
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicListValueAction#intDyadicListAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public List<List<Integer>> intDyadicListAction(String actionName,
			Map<String,Object> input) throws ExecuteSQLException {
		try {
			return this.intDyadicListAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicListValueAction#intDyadicListAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public List<List<Integer>> intDyadicListAction(String actionName,
			Map<String,Object> input, Connection conn) throws ExecuteSQLException {
		return this.intDyadicListAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public List<List<Integer>> intDyadicListAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		List<List<Object>> objects = null;
		try {
			objects = this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
		return new ListMaker<Integer>().transform(objects);
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicListValueAction#longDyadicListAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public List<List<Long>> longDyadicListAction(String actionName,
			Map<String,Object> input) throws ExecuteSQLException {
		try {
			return this.longDyadicListAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicListValueAction#longDyadicListAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public List<List<Long>> longDyadicListAction(String actionName,
			Map<String,Object> input, Connection conn) throws ExecuteSQLException {
		return this.longDyadicListAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public List<List<Long>> longDyadicListAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		List<List<Object>> objects = null;
		try {
			objects = this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
		return new ListMaker<Long>().transform(objects);
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicListValueAction#floatDyadicListAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public List<List<Float>> floatDyadicListAction(String actionName,
			Map<String,Object> input) throws ExecuteSQLException {
		try {
			return this.floatDyadicListAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicListValueAction#floatDyadicListAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public List<List<Float>> floatDyadicListAction(String actionName,
			Map<String,Object> input, Connection conn) throws ExecuteSQLException {
		return this.floatDyadicListAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public List<List<Float>> floatDyadicListAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		List<List<Object>> objects = null;
		try {
			objects = this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
		return new ListMaker<Float>().transform(objects);
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicListValueAction#doubleDyadicListAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public List<List<Double>> doubleDyadicListAction(String actionName,
			Map<String,Object> input) throws ExecuteSQLException {
		try {
			return this.doubleDyadicListAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicListValueAction#doubleDyadicListAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public List<List<Double>> doubleDyadicListAction(String actionName,
			Map<String,Object> input, Connection conn) throws ExecuteSQLException {
		return this.doubleDyadicListAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public List<List<Double>> doubleDyadicListAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		List<List<Object>> objects = null;
		try {
			objects = this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
		return new ListMaker<Double>().transform(objects);
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicListValueAction#booleanDyadicListAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public List<List<Boolean>> booleanDyadicListAction(String actionName,
			Map<String,Object> input) throws ExecuteSQLException {
		try {
			return this.booleanDyadicListAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicListValueAction#booleanDyadicListAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public List<List<Boolean>> booleanDyadicListAction(String actionName,
			Map<String,Object> input, Connection conn) throws ExecuteSQLException {
		return this.booleanDyadicListAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public List<List<Boolean>> booleanDyadicListAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		List<List<Object>> objects = null;
		try {
			objects = this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
		return new ListMaker<Boolean>().transform(objects);
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicListValueAction#stringDyadicListAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public List<List<String>> stringDyadicListAction(String actionName,
			Map<String,Object> input) throws ExecuteSQLException {
		try {
			return this.stringDyadicListAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicListValueAction#stringDyadicListAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public List<List<String>> stringDyadicListAction(String actionName,
			Map<String,Object> input, Connection conn) throws ExecuteSQLException {
		return this.stringDyadicListAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public List<List<String>> stringDyadicListAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		List<List<Object>> objects = null;
		try {
			objects = this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
		return new ListMaker<String>().transform(objects);
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicListValueAction#objDyadicListAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public List<List<Object>> objDyadicListAction(String actionName,
			Map<String,Object> input) throws ExecuteSQLException {
		try {
			return this.objDyadicListAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IDyadicListValueAction#objDyadicListAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public List<List<Object>> objDyadicListAction(String actionName,
			Map<String,Object> input, Connection conn) throws ExecuteSQLException {
		return this.objDyadicListAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public List<List<Object>> objDyadicListAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		try {
			return this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
	}

}
