/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.custom;

import java.sql.Connection;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import com.nonesole.persistence2.connection.exception.OperationsException;
import com.nonesole.persistence2.connection.pool.MultiConnectionPool;
import com.nonesole.persistence2.sql.basic.DefaultBaseAction;
import com.nonesole.persistence2.sql.define.IListValueAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;

/**
 * Implementation class of IListValueAction
 * @author jack lee
 * @version 1.0
 */
public class DefaultListValueAction extends DefaultBaseAction implements IListValueAction {

	private String defaultDatasource = null;

	private MultiConnectionPool pool = null;
	
	private DefaultArrayValueAction arrayValueAction = null;
	
	/**
	 * Constructor
	 */
	public DefaultListValueAction() {
		this(true);
	}

	/**
	 * Constructor
	 * @param isPreparedStatement
	 */
	public DefaultListValueAction(boolean isPreparedStatement) {
		this(null, isPreparedStatement);
	}

	/**
	 * Constructor
	 * @param datasrouceName
	 * @param isPreparedStatement
	 */
	public DefaultListValueAction(String datasrouceName) {
		this(datasrouceName, true);
	}

	/**
	 * Constructor
	 * @param datasrouceName
	 * @param isPreparedStatement
	 */
	public DefaultListValueAction(String datasrouceName,
			boolean isPreparedStatement) {
		super(isPreparedStatement);
		this.defaultDatasource = datasrouceName;
		pool = MultiConnectionPool.getInstance();
		arrayValueAction = new DefaultArrayValueAction(datasrouceName,isPreparedStatement);
	}
	
	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IListValueAction#shortListAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public List<Short> shortListAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.shortListAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IListValueAction#shortListAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public List<Short> shortListAction(String actionName, Map<String,Object> input,
			Connection conn) throws ExecuteSQLException {
		return this.shortListAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public List<Short> shortListAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		Short[] r = arrayValueAction.shortArrayAction(actionName, input, conn, isPrepared);
		if ( r == null ) {
			return new ArrayList<Short>();
		}
		return Arrays.asList(r);
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IListValueAction#intListAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public List<Integer> intListAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.intListAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IListValueAction#intListAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public List<Integer> intListAction(String actionName, Map<String,Object> input,
			Connection conn) throws ExecuteSQLException {
		return this.intListAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public List<Integer> intListAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		Integer[] r = arrayValueAction.intArrayAction(actionName, input, conn, isPrepared);
		if ( r == null ) {
			return new ArrayList<Integer>();
		}
		return Arrays.asList(r);
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IListValueAction#longListAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public List<Long> longListAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.longListAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IListValueAction#longListAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public List<Long> longListAction(String actionName, Map<String,Object> input,
			Connection conn) throws ExecuteSQLException {
		return this.longListAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public List<Long> longListAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		Long[] r = arrayValueAction.longArrayAction(actionName, input, conn, isPrepared);
		if ( r == null ) {
			return new ArrayList<Long>();
		}
		return Arrays.asList(r);
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IListValueAction#floatListAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public List<Float> floatListAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.floatListAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IListValueAction#floatListAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public List<Float> floatListAction(String actionName, Map<String,Object> input,
			Connection conn) throws ExecuteSQLException {
		return this.floatListAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public List<Float> floatListAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		Float[] r = arrayValueAction.floatArrayAction(actionName, input, conn, isPrepared);
		if ( r == null ) {
			return new ArrayList<Float>();
		}
		return Arrays.asList(r);
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IListValueAction#doubleListAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public List<Double> doubleListAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.doubleListAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IListValueAction#doubleListAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public List<Double> doubleListAction(String actionName, Map<String,Object> input,
			Connection conn) throws ExecuteSQLException {
		return this.doubleListAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public List<Double> doubleListAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		Double[] r = arrayValueAction.doubleArrayAction(actionName, input, conn, isPrepared);
		if ( r == null ) {
			return new ArrayList<Double>();
		}
		return Arrays.asList(r);
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IListValueAction#booleanListAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public List<Boolean> booleanListAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.booleanListAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IListValueAction#booleanListAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public List<Boolean> booleanListAction(String actionName, Map<String,Object> input,
			Connection conn) throws ExecuteSQLException {
		return this.booleanListAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public List<Boolean> booleanListAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		Boolean[] r = arrayValueAction.booleanArrayAction(actionName, input, conn, isPrepared);
		if ( r == null ) {
			return new ArrayList<Boolean>();
		}
		return Arrays.asList(r);
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IListValueAction#stringListAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public List<String> stringListAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.stringListAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IListValueAction#stringListAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public List<String> stringListAction(String actionName, Map<String,Object> input,
			Connection conn) throws ExecuteSQLException {
		return this.stringListAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public List<String> stringListAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		String[] r = arrayValueAction.stringArrayAction(actionName, input, conn, isPrepared);
		if ( r == null ) {
			return new ArrayList<String>();
		}
		return Arrays.asList(r);
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IListValueAction#objListAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public List<Object> objListAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.objListAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.IListValueAction#objListAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public List<Object> objListAction(String actionName, Map<String,Object> input,
			Connection conn) throws ExecuteSQLException {
		return this.objListAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public List<Object> objListAction(String actionName,
			Map<String, Object> input, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		Object[] r = arrayValueAction.objArrayAction(actionName, input, conn, isPrepared);
		if ( r == null ) {
			return new ArrayList<Object>();
		}
		return Arrays.asList(r);
	}

}
