/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.custom;

import java.sql.Connection;
import java.util.List;
import java.util.Map;

import com.nonesole.persistence2.config.bean.Action;
import com.nonesole.persistence2.connection.exception.OperationsException;
import com.nonesole.persistence2.connection.pool.MultiConnectionPool;
import com.nonesole.persistence2.sql.ConfigLoader;
import com.nonesole.persistence2.sql.basic.DefaultBaseAction;
import com.nonesole.persistence2.sql.basic.DefaultBasicPreparedAction;
import com.nonesole.persistence2.sql.basic.DefaultBasicStatementAction;
import com.nonesole.persistence2.sql.basic.input.MapInputTransform;
import com.nonesole.persistence2.sql.basic.output.PojoListOutputTransform;
import com.nonesole.persistence2.sql.basic.output.PojoOutputTransform;
import com.nonesole.persistence2.sql.define.IPojoAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;

/**
 * Implementation class of IPojoAction
 * @author jack lee
 * @version 1.0
 * @date 2016-10-31
 */
public class DefaultPojoAction<E> extends DefaultBaseAction implements IPojoAction<E> {

	private String defaultDatasource = null;

	private MultiConnectionPool pool = null;

	private DefaultBasicPreparedAction preparedAction = null;
	
	private DefaultBasicStatementAction statementAction = null;

	/**
	 * Constructor
	 */
	public DefaultPojoAction() {
		this(true);
	}

	/**
	 * Constructor
	 * 
	 * @param isPreparedStatement
	 */
	public DefaultPojoAction(boolean isPreparedStatement) {
		this(null, isPreparedStatement);
	}

	/**
	 * Constructor
	 * 
	 * @param datasrouceName
	 * @param isPreparedStatement
	 */
	public DefaultPojoAction(String datasrouceName) {
		this(datasrouceName, true);
	}

	/**
	 * Constructor
	 * 
	 * @param datasrouceName
	 * @param isPreparedStatement
	 */
	public DefaultPojoAction(String datasrouceName,
			boolean isPreparedStatement) {
		super(isPreparedStatement);
		this.defaultDatasource = datasrouceName;
		pool = MultiConnectionPool.getInstance();
		preparedAction = DefaultBasicPreparedAction.getInstance();
		statementAction = DefaultBasicStatementAction.getInstance();
	}

	@Override
	public  E pojoAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.pojoAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	@Override
	public E pojoAction(String actionName, Map<String,Object> input, Connection conn)
			throws ExecuteSQLException {
		return this.pojoAction(actionName, input, conn, this.isPreparedStatement);
	}

	@SuppressWarnings("unchecked")
	@Override
	public E pojoAction(String actionName, Map<String, Object> input,
			Connection conn, boolean isPrepared) throws ExecuteSQLException {
		Action action = ConfigLoader.getInstance().getAction(actionName);
		MapInputTransform inputTransform = new MapInputTransform(input, isPrepared);
		PojoOutputTransform outTransform = new PojoOutputTransform(action);
		if ( isPrepared ) {	
			try {
				preparedAction.queryAction(action, conn, inputTransform, outTransform);
			} catch (Exception e) {
				throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
			}
		} else {
			try {
				statementAction.queryAction(action, conn, inputTransform, outTransform);
			} catch (Exception e) {
				throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
			}
		}	
		try {
			return (E)outTransform.getObject();
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
	}

	@Override
	public  List<E> pojoListAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.pojoListAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	@Override
	public List<E> pojoListAction(String actionName, Map<String,Object> input,
			Connection conn) throws ExecuteSQLException {
		return this.pojoListAction(actionName, input, conn, this.isPreparedStatement);
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<E> pojoListAction(String actionName, Map<String, Object> input,
			Connection conn, boolean isPrepared) throws ExecuteSQLException {
		Action action = ConfigLoader.getInstance().getAction(actionName);
		MapInputTransform inputTransform = new MapInputTransform(input, isPrepared);
		PojoListOutputTransform outTransform = new PojoListOutputTransform(action);
		if ( isPrepared ) {	
			try {
				preparedAction.queryAction(action, conn, inputTransform, outTransform);
			} catch (Exception e) {
				throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
			}
		} else {
			try {
				statementAction.queryAction(action, conn, inputTransform, outTransform);
			} catch (Exception e) {
				throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
			}
		}	
		try {
			return (List<E>)outTransform.getObject();
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
	}

}
