/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.custom;

import java.sql.Connection;
import java.util.Map;

import com.nonesole.persistence2.config.bean.Action;
import com.nonesole.persistence2.connection.exception.OperationsException;
import com.nonesole.persistence2.connection.pool.MultiConnectionPool;
import com.nonesole.persistence2.sql.ConfigLoader;
import com.nonesole.persistence2.sql.basic.DefaultBaseAction;
import com.nonesole.persistence2.sql.basic.DefaultBasicPreparedAction;
import com.nonesole.persistence2.sql.basic.DefaultBasicStatementAction;
import com.nonesole.persistence2.sql.basic.input.MapInputTransform;
import com.nonesole.persistence2.sql.basic.output.SingleValueOutputTransform;
import com.nonesole.persistence2.sql.define.ISingleValueAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;

/**
 * Implementation class of ISingleValueAction
 * @author jack lee
 * @version 1.0 - 2016-06-18
 */
public class DefaultSingleValueAction extends DefaultBaseAction implements ISingleValueAction {

	private String defaultDatasource = null;
	private MultiConnectionPool pool = null;
	private DefaultBasicPreparedAction preparedAction = null;
	private DefaultBasicStatementAction statementAction = null;
	
	/**
	 * Constructor
	 */
	public DefaultSingleValueAction(){
		this(true);
	}
	
	/**
	 * Constructor
	 * @param isPreparedStatement
	 */
	public DefaultSingleValueAction(boolean isPreparedStatement){
		this(null,isPreparedStatement);
	}
	
	/**
	 * Constructor
	 * @param datasrouceName
	 * @param isPreparedStatement
	 */
	public DefaultSingleValueAction(String datasrouceName){
		this(datasrouceName,true);
	}
	
	/**
	 * Constructor
	 * @param datasrouceName
	 * @param isPreparedStatement
	 */
	public DefaultSingleValueAction(String datasrouceName,boolean isPreparedStatement){
		super(isPreparedStatement);
		this.defaultDatasource = datasrouceName;
		pool = MultiConnectionPool.getInstance();
		preparedAction = DefaultBasicPreparedAction.getInstance();
		statementAction = DefaultBasicStatementAction.getInstance();
	}
	
	/**
	 * The method doAction should be used by all public methods 
	 * of this class. The return value is single-value.
	 * 
	 * @param actionName
	 * @param input
	 * @param conn
	 * @param isPrepared
	 * @return
	 * @throws Exception
	 */
	protected Object doAction(String actionName, Map<String,Object> input, Connection conn, boolean isPrepared) 
	throws Exception{
		Action action = ConfigLoader.getInstance().getAction(actionName);
		MapInputTransform inputTransform = new MapInputTransform(input, isPrepared);
		SingleValueOutputTransform outTransform = new SingleValueOutputTransform(action);
		if ( isPrepared ) {	
			preparedAction.queryAction(action, conn, inputTransform, outTransform);
		} else {
			statementAction.queryAction(action, conn, inputTransform, outTransform);
		}	
		return outTransform.getObject();
	}
	
	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.ISingleValueAction#shortAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public Short shortAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.shortAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.ISingleValueAction#shortAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public Short shortAction(String actionName, Map<String,Object> input, Connection conn)
			throws ExecuteSQLException {
		return this.shortAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public Short shortAction(String actionName, Map<String, Object> input,
			Connection conn, boolean isPrepared) throws ExecuteSQLException {
		Object object = null;
		try {
			object = this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
		if ( object != null ) {
			return (Short)object;
		}
		return 0;
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.ISingleValueAction#intAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public Integer intAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.intAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.ISingleValueAction#intAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public Integer intAction(String actionName, Map<String,Object> input, Connection conn)
			throws ExecuteSQLException {
		return this.intAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public Integer intAction(String actionName, Map<String, Object> input,
			Connection conn, boolean isPrepared) throws ExecuteSQLException {
		Object object = null;
		try {
			object = this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
		if ( object != null ) {
			return (Integer)object;
		}
		return 0;
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.ISingleValueAction#longAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public Long longAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.longAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.ISingleValueAction#longAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public Long longAction(String actionName, Map<String,Object> input, Connection conn)
			throws ExecuteSQLException {
		return this.longAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public Long longAction(String actionName, Map<String, Object> input,
			Connection conn, boolean isPrepared) throws ExecuteSQLException {
		Object object = null;
		try {
			object = this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
		if ( object != null ) {
			return (Long)object;
		}
		return 0l;
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.ISingleValueAction#floatAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public Float floatAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.floatAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.ISingleValueAction#floatAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public Float floatAction(String actionName, Map<String,Object> input, Connection conn)
			throws ExecuteSQLException {
		return this.floatAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public Float floatAction(String actionName, Map<String, Object> input,
			Connection conn, boolean isPrepared) throws ExecuteSQLException {
		Object object = null;
		try {
			object = this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
		if ( object != null ) {
			return (Float)object;
		}
		return 0f;
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.ISingleValueAction#doubleAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public Double doubleAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.doubleAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.ISingleValueAction#doubleAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public Double doubleAction(String actionName, Map<String,Object> input,
			Connection conn) throws ExecuteSQLException {
		return this.doubleAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public Double doubleAction(String actionName, Map<String, Object> input,
			Connection conn, boolean isPrepared) throws ExecuteSQLException {
		Object object = null;
		try {
			object = this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
		if ( object != null ) {
			return (Double)object;
		}
		return 0d;
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.ISingleValueAction#booleanAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public Boolean booleanAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.booleanAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.ISingleValueAction#booleanAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public Boolean booleanAction(String actionName, Map<String,Object> input,
			Connection conn) throws ExecuteSQLException {
		return this.booleanAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public Boolean booleanAction(String actionName, Map<String, Object> input,
			Connection conn, boolean isPrepared) throws ExecuteSQLException {
		Object object = null;
		try {
			object = this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
		if ( object != null ) {
			return (Boolean)object;
		}
		return false;
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.ISingleValueAction#stringAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public String stringAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.stringAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.ISingleValueAction#stringAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public String stringAction(String actionName, Map<String,Object> input,
			Connection conn) throws ExecuteSQLException {
		return this.stringAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public String stringAction(String actionName, Map<String, Object> input,
			Connection conn, boolean isPrepared) throws ExecuteSQLException {
		Object object = null;
		try {
			object = this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
		if ( object != null ) {
			return (String)object;
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.ISingleValueAction#objAction(java.lang.String, java.lang.Object[])
	 */
	@Override
	public Object objAction(String actionName, Map<String,Object> input)
			throws ExecuteSQLException {
		try {
			return this.objAction(actionName, input, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.nonesole.persistence2.sql.ISingleValueAction#objAction(java.lang.String, java.lang.Object[], java.sql.Connection)
	 */
	@Override
	public Object objAction(String actionName, Map<String,Object> input, Connection conn)
			throws ExecuteSQLException {
		return this.objAction(actionName, input, conn, this.isPreparedStatement);
	}

	@Override
	public Object objAction(String actionName, Map<String, Object> input,
			Connection conn, boolean isPrepared) throws ExecuteSQLException {
		try {
			return this.doAction(actionName, input, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.getLocalizedMessage(),e.fillInStackTrace());
		}
	}
	
}
