/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.custom;

import java.sql.Connection;
import java.util.List;
import java.util.Map;

import com.nonesole.persistence2.config.bean.Action;
import com.nonesole.persistence2.config.bean.InputObject;
import com.nonesole.persistence2.connection.exception.OperationsException;
import com.nonesole.persistence2.connection.pool.MultiConnectionPool;
import com.nonesole.persistence2.sql.ConfigLoader;
import com.nonesole.persistence2.sql.basic.DefaultBaseAction;
import com.nonesole.persistence2.sql.basic.DefaultBasicPreparedAction;
import com.nonesole.persistence2.sql.basic.DefaultBasicStatementAction;
import com.nonesole.persistence2.sql.basic.input.InputTransform;
import com.nonesole.persistence2.sql.basic.input.MapInputTransform;
import com.nonesole.persistence2.sql.basic.input.PojoInputTransform;
import com.nonesole.persistence2.sql.basic.input.PojoListInputTransform;
import com.nonesole.persistence2.sql.basic.input.SimpleListInputTransform;
import com.nonesole.persistence2.sql.define.IVoidAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;

/**
 * Implementation class of IVoidAction
 * @author jack lee
 * @version 2.0
 * @date 2016-10-04
 */
public class DefaultVoidAction extends DefaultBaseAction implements IVoidAction {

	private String defaultDatasource = null;
	private MultiConnectionPool pool = null;
	private DefaultBasicPreparedAction preparedAction = null;
	private DefaultBasicStatementAction statementAction = null;
	
	/**
	 * Constructor
	 */
	public DefaultVoidAction() {
		this(true);
	}

	/**
	 * Constructor
	 * 
	 * @param isPreparedStatement
	 */
	public DefaultVoidAction(boolean isPreparedStatement) {
		this(null, isPreparedStatement);
	}

	/**
	 * Constructor
	 * 
	 * @param datasrouceName
	 * @param isPreparedStatement
	 */
	public DefaultVoidAction(String datasrouceName) {
		this(datasrouceName, true);
	}

	/**
	 * Constructor
	 * 
	 * @param datasrouceName
	 * @param isPreparedStatement
	 */
	public DefaultVoidAction(String datasrouceName, boolean isPreparedStatement) {
		super(isPreparedStatement);
		this.defaultDatasource = datasrouceName;
		pool = MultiConnectionPool.getInstance();
		preparedAction = DefaultBasicPreparedAction.getInstance();
		statementAction = DefaultBasicStatementAction.getInstance();
	}

	@Override
	public void voidAction(String actionName, Object object)
			throws ExecuteSQLException {
		try {
			Action act = ConfigLoader.getInstance().getAction(actionName);
			act.setTransaction(true);
			this.voidAction(actionName, object, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e.getMessage(),e.fillInStackTrace());
		}
	}

	@Override
	public void voidAction(String actionName, Object object, Connection conn)
			throws ExecuteSQLException {
		this.voidAction(actionName, object, conn, this.isPreparedStatement);
	}

	@SuppressWarnings("unchecked")
	@Override
	public void voidAction(String actionName, Object object, Connection conn,
			boolean isPrepared) throws ExecuteSQLException {
		Action action = ConfigLoader.getInstance().getAction(actionName);
		InputObject input = action.getInput();
		InputTransform inputTransform = null;
		if ( object == null ) {
			throw new ExecuteSQLException("Input value is null.");
		}
		if ( input != null ) {
			if ( InputObject.MAP.equals(input.getProperty()) ) {
				inputTransform = new MapInputTransform((Map<String,Object>)object, isPrepared);
			} else if ( InputObject.POJO.equals(input.getProperty()) ) {
				inputTransform = new PojoInputTransform(object);
			} else if ( InputObject.POJO_LIST.equals(input.getProperty()) ) {
				inputTransform = new PojoListInputTransform((List<Object>)object);
			} else if ( InputObject.SIMPLE_LIST.equals(input.getProperty()) ) {
				inputTransform = new SimpleListInputTransform((List<Object>)object);
			} else {
				throw new ExecuteSQLException("Input property is wrong.(Property:"+input.getProperty()+")");
			}
		}
		
		boolean partOfTransaction = !action.isTransaction();
		
		if ( isPrepared ) {	
			try {
				preparedAction.executeAction(action, conn, partOfTransaction, inputTransform);
			} catch (Exception e) {
				throw new ExecuteSQLException(e.getMessage(),e.fillInStackTrace());
			}
		} else {
			try {
				statementAction.executeAction(action, conn, partOfTransaction, inputTransform);
			} catch (Exception e) {
				throw new ExecuteSQLException(e.getMessage(),e.fillInStackTrace());
			}
		}	
	}
}
