/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.define;

import java.sql.Connection;
import java.util.List;

import com.nonesole.persistence2.sql.exception.ExecuteSQLException;

/**
 * SQL operation interface
 * 
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-11
 */
public interface IOrmAction  {

	/**
	 * Insert one object to database
	 * @param obj 
	 * @param conn
	 * @param isPrepared
	 * @return object
	 * */
	public Object insert(Object obj,Connection conn,boolean isPrepared) throws ExecuteSQLException;
	
	/**
	 * Update one object in database
	 * @param obj 
	 * @param conn
	 * @param isPrepared
	 * @return object
	 * */
	public Object update(Object obj,Connection conn,boolean isPrepared) throws ExecuteSQLException;
	
	/**
	 * Delete one object in database
	 * @param obj 
	 * @param conn
	 * @param isPrepared
	 * */
	public void delete(Object obj,Connection conn,boolean isPrepared) throws ExecuteSQLException;
	
	/**
	 * Insert list of objects to database
	 * @param list of obj 
	 * @param conn
	 * @param isPrepared
	 * @return list of object
	 * */
	public List<?> batchInsert(List<?> obj,Connection conn,boolean isPrepared) throws ExecuteSQLException;
	
	/**
	 * Update list of objects in database
	 * @param list of obj 
	 * @param conn
	 * @param isPrepared
	 * @return list of object
	 * */
	public List<?> batchUpdate(List<?> obj,Connection conn,boolean isPrepared) throws ExecuteSQLException;
	
	/**
	 * Delete list of objects in database
	 * @param list of obj 
	 * @param conn
	 * @param isPrepared
	 * */
	public void batchDelete(List<?> obj,Connection conn,boolean isPrepared) throws ExecuteSQLException;
	
	
	/**
	 * Insert one object to database
	 * @param obj 
	 * @param conn
	 * @return object
	 * */
	public Object insert(Object obj,Connection conn) throws ExecuteSQLException;
	
	/**
	 * Update one object in database
	 * @param obj 
	 * @param conn
	 * @return object
	 * */
	public Object update(Object obj,Connection conn) throws ExecuteSQLException;
	
	/**
	 * Delete one object in database
	 * @param obj 
	 * @param conn
	 * */
	public void delete(Object obj,Connection conn) throws ExecuteSQLException;
		
	/**
	 * Search objects in database
	 * @param sql 
	 * @param clazz - Class of object
	 * @param conn
	 * @return object
	 * */
	public List<?> query(String sql,Class<?> clazz,Connection conn) throws ExecuteSQLException;
		
	
	/**
	 * Execute sql (it would be insert,update,delete sql)
	 * @param sql
	 * @param conn
	 * */
	public void execute(String sql,Connection conn) throws ExecuteSQLException;
	
	/**
	 * Execute sql (it would be insert,update,delete sql) using default connection pool
	 * @param sql
	 * @param conn
	 * */
	public void batchExecute(List<String> list,Connection conn) throws ExecuteSQLException;
	
	/**
	 * Insert list of objects to database
	 * @param list of obj 
	 * @param conn
	 * @return list of object
	 * */
	public List<?> batchInsert(List<?> obj,Connection conn) throws ExecuteSQLException;
	
	/**
	 * Update list of objects in database
	 * @param list of obj 
	 * @param conn
	 * @return list of object
	 * */
	public List<?> batchUpdate(List<?> obj,Connection conn) throws ExecuteSQLException;
	
	/**
	 * Delete list of objects in database
	 * @param list of obj 
	 * @param conn
	 * */
	public void batchDelete(List<?> obj,Connection conn) throws ExecuteSQLException;
	
	/**
	 * Insert one object to database using default connection pool
	 * @param obj 
	 * @param conn
	 * @return object
	 * */
	public Object insert(Object obj) throws ExecuteSQLException;
	
	/**
	 * Update one object in database using default connection pool
	 * @param obj 
	 * @param conn
	 * @return object
	 * */
	public Object update(Object obj) throws ExecuteSQLException;
	
	/**
	 * Delete one object in database using default connection pool
	 * @param obj 
	 * @param conn
	 * */
	public void delete(Object obj) throws ExecuteSQLException;
		
	/**
	 * Search objects in database using default connection pool
	 * @param sql 
	 * @param clazz - Class of object
	 * @param conn
	 * @return object
	 * */
	public List<?> query(String sql,Class<?> clazz) throws ExecuteSQLException;
		
	
	/**
	 * Execute sql (it would be insert,update,delete sql) using default connection pool
	 * @param sql
	 * @param conn
	 * */
	public void execute(String sql) throws ExecuteSQLException;
	

	/**
	 * Insert list of objects to database using default connection pool
	 * @param list of obj 
	 * @param conn
	 * @return list of object
	 * */
	public List<?> batchInsert(List<?> obj) throws ExecuteSQLException;
	
	/**
	 * Update list of objects in database using default connection pool
	 * @param list of obj 
	 * @param conn
	 * @return list of object
	 * */
	public List<?> batchUpdate(List<?> obj) throws ExecuteSQLException;
	
	/**
	 * Delete list of objects in database using default connection pool
	 * @param list of obj 
	 * @param conn
	 * */
	public void batchDelete(List<?> obj) throws ExecuteSQLException;
	
	/**
	 * Execute sql (it would be insert,update,delete sql) using default connection pool
	 * @param sql
	 * @param conn
	 * */
	public void batchExecute(List<String> list) throws ExecuteSQLException;
	
}
