/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.orm;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import com.nonesole.persistence2.config.CommonTools;
import com.nonesole.persistence2.connection.exception.OperationsException;
import com.nonesole.persistence2.connection.pool.MultiConnectionPool;
import com.nonesole.persistence2.sql.basic.DefaultBaseAction;
import com.nonesole.persistence2.sql.define.IOrmAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;

/**
 * Implementation class of IOrmAction
 * @author jack lee
 * @version 1.0
 */
public class DefaultOrmAction extends DefaultBaseAction implements IOrmAction{
	
	protected String defaultDatasource = null;
	
	protected MultiConnectionPool pool = null;
	
	/**
	 * Constructor
	 */
	public DefaultOrmAction(){
		this(true);
	}
	
	/**
	 * Constructor
	 * @param isPreparedStatement
	 */
	public DefaultOrmAction(boolean isPreparedStatement){
		this(null,isPreparedStatement);
	}
	
	/**
	 * Constructor
	 * @param datasrouceName
	 * @param isPreparedStatement
	 */
	public DefaultOrmAction(String datasrouceName){
		this(datasrouceName,true);
	}
	
	/**
	 * Constructor
	 * @param datasrouceName
	 * @param isPreparedStatement
	 */
	public DefaultOrmAction(String datasrouceName,boolean isPreparedStatement){
		super(isPreparedStatement);
		this.defaultDatasource = datasrouceName;
		pool = MultiConnectionPool.getInstance();
	}

	/**
	 * Check whether the object is null.
	 * @param obj
	 * @param conn
	 */
	protected void checkNull(Object obj, Connection conn)
			throws ExecuteSQLException{
		if ( CommonTools.isNull(obj) ) {
			throw new ExecuteSQLException("Object is null.");	
		}
		if ( CommonTools.isNull(conn) ) {
			throw new ExecuteSQLException("Connection is null.");	
		}
	}
	
	@Override
	public Object insert(Object obj, Connection conn)
			throws ExecuteSQLException {
		return this.insert(obj, conn, this.isPreparedStatement);
	}

	@Override
	public Object insert(Object obj, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		this.checkNull(obj, conn);
		try {
			this.insertWithoutCommit(obj, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.fillInStackTrace());
		}
		return obj;
	}

	@Override
	public Object update(Object obj, Connection conn)
			throws ExecuteSQLException {
		return this.update(obj, conn, this.isPreparedStatement);
	}

	@Override
	public Object update(Object obj, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		this.checkNull(obj, conn);
		try {
			this.updateWithoutCommit(obj, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.fillInStackTrace());
		}
		return obj;
	}

	@Override
	public void delete(Object obj, Connection conn) throws ExecuteSQLException {
		this.delete(obj, conn, this.isPreparedStatement);
	}

	@Override
	public void delete(Object obj, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		this.checkNull(obj, conn);
		try {
			this.deleteWithoutCommit(obj, conn, isPrepared);
		} catch (Exception e) {
			throw new ExecuteSQLException(e.fillInStackTrace());
		}
	}
	
	@Override
	public List<Object> query(String sql, Class<?> clazz, Connection conn)
			throws ExecuteSQLException {
		return super.query(sql, clazz, conn);
	}

	public void execute(String sql, Connection conn) throws ExecuteSQLException {
		super.execute(sql, conn);
	}

	public void executeSave(String sql, Connection conn, Object obj) throws ExecuteSQLException {
		super.executeSave(sql, conn, obj);
	}
	
	@Override
	public List<Object> batchInsert(List<?> obj, Connection conn)
			throws ExecuteSQLException {
		return this.batchInsert(obj, conn, this.isPreparedStatement);
	}
	
	@Override
	public List<Object> batchInsert(List<?> obj, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		this.checkNull(obj, conn);
		DefaultBaseAction.commitConnection(conn, false);
		List<Object> l = new ArrayList<Object>();
		for (Object o : obj) {
			try {
				l.add(this.insertWithoutCommit(o, conn, isPrepared));
			} catch (Exception e) {
				throw new ExecuteSQLException(e.fillInStackTrace());
			}
		}
		DefaultBaseAction.commitConnection(conn, true);
		return l;
	}

	@Override
	public List<Object> batchUpdate(List<?> obj, Connection conn)
			throws ExecuteSQLException {
		return this.batchUpdate(obj, conn, this.isPreparedStatement);
	}
	
	@Override
	public List<Object> batchUpdate(List<?> obj, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		this.checkNull(obj, conn);
		DefaultBaseAction.commitConnection(conn, false);
		List<Object> l = new ArrayList<Object>();
		for (Object o : obj) {
			try {
				l.add(this.updateWithoutCommit(o, conn, isPrepared));
			} catch (Exception e) {
				throw new ExecuteSQLException(e.fillInStackTrace());
			}
		}
		DefaultBaseAction.commitConnection(conn, true);
		return l;
	}

	@Override
	public void batchDelete(List<?> obj, Connection conn)
			throws ExecuteSQLException {
		this.batchDelete(obj, conn, this.isPreparedStatement);
	}

	@Override
	public void batchDelete(List<?> obj, Connection conn, boolean isPrepared)
			throws ExecuteSQLException {
		this.checkNull(obj, conn);
		DefaultBaseAction.commitConnection(conn, false);
		for ( Object o : obj ) {
			try {
				this.deleteWithoutCommit(o, conn, isPrepared);
			} catch (Exception e) {
				throw new ExecuteSQLException(e.fillInStackTrace());
			}
		}
		DefaultBaseAction.commitConnection(conn, true);
	}
	
	@Override
	public Object insert(Object obj) throws ExecuteSQLException {
		try {
			Connection conn = this.pool.getConnection(this.defaultDatasource);
			DefaultBaseAction.commitConnection(conn, false);//���÷��Զ��ύ
			Object robj = this.insert(obj, conn);
			DefaultBaseAction.commitConnection(conn, true);
			return robj;
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	@Override
	public Object update(Object obj) throws ExecuteSQLException {
		try {
			Connection conn = this.pool.getConnection(this.defaultDatasource);
			DefaultBaseAction.commitConnection(conn, false);//���÷��Զ��ύ
			Object robj = this.update(obj, conn);
			DefaultBaseAction.commitConnection(conn, true);
			return robj;
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	@Override
	public void delete(Object obj) throws ExecuteSQLException {
		try {
			Connection conn = this.pool.getConnection(this.defaultDatasource);
			DefaultBaseAction.commitConnection(conn, false);//���÷��Զ��ύ
			this.delete(obj, conn);
			DefaultBaseAction.commitConnection(conn, true);
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	@Override
	public List<Object> query(String sql, Class<?> clazz)
			throws ExecuteSQLException {
		try {
			return this.query(sql, clazz, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	@Override
	public void execute(String sql) throws ExecuteSQLException {
		try {
			this.execute(sql, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	@Override
	public List<Object> batchInsert(List<?> obj)
			throws ExecuteSQLException {
		try {
			return this.batchInsert(obj, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	@Override
	public List<Object> batchUpdate(List<?> obj)
			throws ExecuteSQLException {
		try {
			return this.batchUpdate(obj, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	@Override
	public void batchDelete(List<?> obj) throws ExecuteSQLException {
		try {
			this.batchDelete(obj, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	@Override
	public void batchExecute(List<String> list) throws ExecuteSQLException {
		try {
			this.batchExecute(list, this.pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException(e);
		}
	}

	@Override
	public void batchExecute(List<String> list, Connection conn)
			throws ExecuteSQLException {
		this.checkNull(list, conn);
		if ( null != list && !list.isEmpty() ) {
			Statement stmt = null;
			DefaultBaseAction.commitConnection(conn, false);			
			for ( String sql : list ) {
				if ( null == sql || "".equals(sql) )
					throw new ExecuteSQLException("Sql is null");
				
				DefaultBaseAction.preventInjectionAttacks(sql);
				try {
					stmt = (Statement) conn.createStatement();
					stmt.execute(sql);
				} catch (SQLException e) {
					DefaultBaseAction.connRollback(conn, e);
				} 
			}
			DefaultBaseAction.commitConnection(conn, true);
		}
	}

}
