/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.orm;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.config.IOrmMaker;
import com.nonesole.persistence2.config.ParserException;
import com.nonesole.persistence2.config.bean.Action;
import com.nonesole.persistence2.config.bean.InputMapping;
import com.nonesole.persistence2.config.bean.InputObject;
import com.nonesole.persistence2.config.bean.OSMObject;
import com.nonesole.persistence2.connection.exception.OperationsException;
import com.nonesole.persistence2.connection.pool.MultiConnectionPool;
import com.nonesole.persistence2.sql.ConfigLoader;
import com.nonesole.persistence2.sql.basic.DefaultBaseAction;
import com.nonesole.persistence2.sql.basic.DefaultBasicPreparedAction;
import com.nonesole.persistence2.sql.define.IOrmPlusAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;

/**
 * Implementation class of IOrmPlusAction and IOrmMaker
 * @author jack lee
 * @version 1.0
 */
public class DefaultOrmPlusAction extends DefaultOrmAction implements IOrmPlusAction,IOrmMaker {
	
	private Logger logger = null;
	
	/**
	 * Constructor
	 */
	public DefaultOrmPlusAction(){
		this(true);
	}
	
	/**
	 * Constructor
	 * @param isPreparedStatement
	 */
	public DefaultOrmPlusAction(boolean isPreparedStatement){
		this(null,isPreparedStatement);
	}
	
	/**
	 * Constructor
	 * @param datasrouceName
	 * @param isPreparedStatement
	 */
	public DefaultOrmPlusAction(String datasrouceName){
		this(datasrouceName,true);
	}
	
	/**
	 * Constructor
	 * @param datasrouceName
	 * @param isPreparedStatement
	 */
	public DefaultOrmPlusAction(String datasrouceName,boolean isPreparedStatement){
		super(isPreparedStatement);
		this.defaultDatasource = datasrouceName;
		pool = MultiConnectionPool.getInstance();
		logger = Logger.getInstance();
	}

	@Override
	public void doAction(String actionName, Object obj)
			throws ExecuteSQLException {
		try {
			this.doAction(actionName, obj, pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			logger.error("[DefaultOrmPlusAction:adAction]SQL error.", e);
		}
	}

	@Override
	public void doBatchAction(String actionName, List<Object> list)
			throws ExecuteSQLException {
		try {
			this.doBatchAction(actionName, list, pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			logger.error("[DefaultOrmPlusAction:doBatchAction]SQL error.", e);
		}
	}

	@Override
	public List<Object> doQueryAction(String actionName, Object obj)
			throws ExecuteSQLException {
		try {
			return this.doQueryAction(actionName, obj, pool.getConnection(this.defaultDatasource));
		} catch (OperationsException e) {
			throw new ExecuteSQLException("[DefaultOrmPlusAction:doQueryAction]SQL error.",e.fillInStackTrace());
		}
	}

	@Override
	public void doAction(String actionName, Object obj, Connection conn)
			throws ExecuteSQLException {
		this.doAction(actionName, obj, conn, this.isPreparedStatement);
	}

	@Override
	public void doAction(String actionName, Object object, Connection conn,
			boolean isPrepared) throws ExecuteSQLException {
		Action action = ConfigLoader.getInstance().getAction(actionName);
		String sql = null;
		try {
			InputObject input = action.getInput();
			if ( input == null ) {
				throw new ParserException("Input is null.");
			}
			if ( InputObject.ORM_POJO.equals(input.getProperty()) ) {
				Map<String,Object> fieldValueMap = action.checkPojoParam(object);
				if ( isPrepared ) {
					try {
						List<Object> values = new ArrayList<Object>();
						List<Class<?>> types = new ArrayList<Class<?>>();
						fieldValueMap.put(IOrmMaker.TYPES, types);
						fieldValueMap.put(IOrmMaker.VALUES, values);
						sql = makeSQL(object,fieldValueMap,action,isPrepared);
						PreparedStatement st = DefaultBasicPreparedAction.getPreparedStatement(conn,sql);
						DefaultBasicPreparedAction.setValues(st, values, types, null,1);
						if ( sql.startsWith("insert") ) {
							DefaultBasicPreparedAction.executeSave(st, object);	
						} else {
							DefaultBasicPreparedAction.execute(st);	
						}
						DefaultBaseAction.connRelease(conn,null,st);
					} catch (Exception e) {
						throw new ParserException("Making sql in orm-model has error.",e.fillInStackTrace());
					}
				} else {
					sql = makeSQL(object,fieldValueMap,action,isPrepared);
					if ( sql.startsWith("insert") ) {
						this.executeSave(sql, conn, object);
					} else {
						this.execute(sql, conn);
					}

				}
			} else {
				throw new ParserException("This is not an orm action, please change the right one.");
			}
		} catch (Exception e) {
			DefaultBaseAction.connRollback(conn, e);
			throw new ExecuteSQLException("[DefaultOrmPlusAction:doAction]",e.fillInStackTrace());
		}
		
	}

	@Override
	public void doBatchAction(String actionName, List<Object> list,
			Connection conn) throws ExecuteSQLException {
		Action act = ConfigLoader.getInstance().getAction(actionName);
		try {
			DefaultBaseAction.commitConnection(conn, false);
			List<String> sqlList = act.list2SQL(list);
			if ( sqlList != null ) {
				for ( String sql : sqlList ) {
					DefaultBaseAction.executeNotCommit(sql, conn);
				}
				DefaultBaseAction.commitConnection(conn, true);
			}
		} catch (Exception e) {
			DefaultBaseAction.connRollback(conn, e);
		}
	}
	
	@Override
	public void doBatchAction(String actionName, List<Object> list,
			Connection conn, boolean isPrepared) throws ExecuteSQLException {
		Action action = ConfigLoader.getInstance().getAction(actionName);
		InputObject input = action.getInput();
		if ( input == null ) {
			throw new ExecuteSQLException("Input is null.");
		}
		if ( list == null || list.isEmpty() ) {
			throw new ExecuteSQLException("The object-list is null.");
		}
		try {
			logger.debug("DefaultOrmPlusAction:doBatchAction()",null);	
			if ( InputObject.ORM_POJO_LIST.equals(input.getProperty()) ) {
				Map<String,Object> fieldValueMap = null;
				DefaultBaseAction.commitConnection(conn, false);
				String sql = null;
				for ( Object object : list ) {
					fieldValueMap = action.checkPojoParam(object);
					
					if ( isPrepared ) {
						try {
							List<Object> values = new ArrayList<Object>();
							List<Class<?>> types = new ArrayList<Class<?>>();
							fieldValueMap.put(IOrmMaker.TYPES, types);
							fieldValueMap.put(IOrmMaker.VALUES, values);
							sql = makeSQL(object,fieldValueMap,action,isPrepared);
							
							PreparedStatement st = DefaultBasicPreparedAction.getPreparedStatement(conn,sql);
							DefaultBasicPreparedAction.setValues(st, values, types, null,1);
							DefaultBasicPreparedAction.execute(st);	
						} catch (Exception e) {
							throw new ParserException("Making sql in orm-model has error.",e.fillInStackTrace());
						}
					} else {
						sql = makeSQL(object,fieldValueMap,action,isPrepared);
						executeNotCommit(sql, conn);
					}
				}
				DefaultBaseAction.commitConnection(conn, true);
				DefaultBaseAction.connRelease(conn,null,null);
				
			} else if ( InputObject.SIMPLE_LIST.equals(input.getProperty()) ){
				if ( input.getMappings() == null || input.getMappings().isEmpty() ) {
					throw new ParserException("The input mapping is null.");
				}
				if ( input.getMappings().size() != 1 ) {
					throw new ParserException("The input mapping is more than one object.");
				}
				DefaultBaseAction.commitConnection(conn, false);
				List<String> sqlList = null;
				InputMapping m = input.getMappings().get(0);
				for ( Object object : list ) {
					Map<String,Object> valueMap = new HashMap<String,Object>();
					action.checkSimpleParam(object);
					valueMap.put(m.getTargetName(), object);
					sqlList = action.getSqls();
					for ( String sql : sqlList ) {
						if ( isPrepared ) {
							try {
								PreparedStatement st = DefaultBasicPreparedAction.getPreparedStatement(conn,sql);
								Set<Entry<String,Object>> fieldValueSet = valueMap.entrySet();
								Object value = null;
								List<Object> values = new ArrayList<Object>();
								List<Class<?>> types = new ArrayList<Class<?>>();
								for ( Entry<String,Object> en : fieldValueSet ) {
									value = en.getValue();
									if ( value != null ) {
										values.add(value);
										types.add(value.getClass());
									}
								}
								DefaultBasicPreparedAction.setValues(st, values, types, null,1);
								DefaultBasicPreparedAction.execute(st);	
							} catch (Exception e) {
								throw new ParserException("Making sql in orm-model has error.",e.fillInStackTrace());
							}
						} else {
							sql = sql.replace(m.getTargetName(), (String)object);
							executeNotCommit(sql, conn);
						}
					}
				}
				DefaultBaseAction.commitConnection(conn, true);
				DefaultBaseAction.connRelease(conn,null,null);
			} else {
				throw new ParserException("The input object has wrong type.");
			}
			
		} catch (Exception e) {
			DefaultBaseAction.connRollback(conn, e);
			throw new ExecuteSQLException("[DefaultOrmPlusAction:doBatchAction]",e.fillInStackTrace());
		}
	}
	

	@Override
	public List<Object> doQueryAction(String actionName, Object obj,
			Connection conn) throws ExecuteSQLException {
		Action act = ConfigLoader.getInstance().getAction(actionName);
		try {
			String sql = act.orm2SQL(this, obj);
			return this.query(sql, obj.getClass(), conn);
		} catch (Exception e) {
			logger.error("[DefaultOrmPlusAction:doQueryAction]That transformation orm to sql is error.", e);
		}
		return null;
	}

	@Override
	public String makeSQL(Object sourceObject,Map<String, Object> fieldValueMap,Action action,boolean isPrepared) throws Exception {
		if ( sourceObject == null || fieldValueMap == null || action == null) {
			return "";
		}
		
		OSMObject osm = ConfigLoader.getInstance()
				.getObjectSqlMapping(sourceObject.getClass().getName());
		if ( null == osm ) {
			throw new ExecuteSQLException("ObjectSqlMapping of "+ sourceObject.getClass().getName() +" is null.");
		}
		
		if ( Action.METHOD_INSERT.equals(action.getOrmAction()) ) {
			if ( !isPrepared ) {
				return this.makeInsertSQL(fieldValueMap, osm);
			} else {
				return this.makePreparedInsertSQL(fieldValueMap, osm);
			}
		} else if (Action.METHOD_UPDATE.equals(action.getOrmAction()) ) {
			if ( !isPrepared ) {
				return this.makeUpdateSQL(fieldValueMap, osm, action);
			} else {
				return this.makePreparedUpdateSQL(fieldValueMap, osm, action);
			}
		} else if (Action.METHOD_DELETE.equals(action.getOrmAction()) ) {
			if ( !isPrepared ) {
				return this.makeDeleteSQL(fieldValueMap, osm, action);
			} else {
				return this.makePreparedDeleteSQL(fieldValueMap, osm, action);
			}
		} else if (Action.METHOD_SELECT.equals(action.getOrmAction()) ) {
			if ( !isPrepared ) {
				return this.makeSelectSQL(fieldValueMap, osm, action);
			} else {
				return this.makePreparedSelectSQL(fieldValueMap, osm, action);
			}
		} 
		
		return "";
	}

	/**
	 * Assemble SQL for insert action.
	 * @param fieldValueMap
	 * @param osm
	 * @return
	 * @throws Exception
	 */
	private String makeInsertSQL(Map<String, Object> fieldValueMap,OSMObject osm) throws Exception {
		
		StringBuilder dbField = new StringBuilder();
		StringBuilder values = new StringBuilder();
		Object value = null;
		
		Set<Entry<String,Object>> fieldValueSet = fieldValueMap.entrySet();
		for ( Entry<String,Object> en : fieldValueSet ) {
			value = en.getValue();
			if ( value != null ) {
				dbField.append(en.getKey()).append(",");
				handleCharacter(value.getClass(),values,value);
				values.append(",");	
			}
		}
		int point = dbField.lastIndexOf(",");
		dbField.delete(point, point+1);
		point = values.lastIndexOf(",");
		values.delete(point, point+1);
		
		//sql
		StringBuilder sql = new StringBuilder();
		sql.append("insert into")
			.append(" ")
			.append(osm.getTableName())
			.append("(")
			.append(dbField.toString())
			.append(") values (")
			.append(values.toString())
			.append(")");
		
		return sql.toString();
	}
	
	/**
	 * Assemble SQL for update action.
	 * @param fieldValueMap
	 * @param action
	 * @return
	 * @throws Exception
	 */
	private String makeUpdateSQL(Map<String, Object> fieldValueMap,OSMObject osm, Action action) throws Exception {
		
		StringBuilder set = new StringBuilder();
		StringBuilder where = new StringBuilder(); 
		StringBuilder temp = null;
		Object value = null;
		
		Set<Entry<String,Object>> fieldValueSet = fieldValueMap.entrySet();
		for ( Entry<String,Object> en : fieldValueSet ) {
			value = en.getValue();
			if ( value != null ) {
				temp = new StringBuilder();
				temp.append(en.getKey()).append("=");
				handleCharacter(value.getClass(),temp,value);
				if ( action.isWhereParameter(en.getKey()) ) {
					where.append(temp).append(" and ");
				} else {
					set.append(temp).append(",");
				}
			}
		}
		int point = set.lastIndexOf(",");
		set.delete(point, point+1);
		point = where.lastIndexOf(" and ");
		where.delete(point, point+6);
		
		if( "".equals(where) ) {
			return "";
		}
		
		StringBuilder sql = new StringBuilder();
		sql.append("update")
			.append(" ")
			.append(osm.getTableName())
			.append(" ")
			.append("set")
			.append(" ")
			.append(set)
			.append(" ")
			.append("where")
			.append(" ")
			.append(where);
		
		return sql.toString();
	}
	
	/**
	 * Assemble SQL for delete action.
	 * @param fieldValueMap
	 * @param action
	 * @return
	 * @throws Exception
	 */
	private String makeDeleteSQL(Map<String, Object> fieldValueMap,OSMObject osm, Action action) throws Exception {
		
		Object value = null;
		StringBuilder where = new StringBuilder(); 
		StringBuilder temp = null;
		
		Set<Entry<String,Object>> fieldValueSet = fieldValueMap.entrySet();
		for ( Entry<String,Object> en : fieldValueSet ) {
			value = en.getValue();
			if ( value != null ) {
				temp = new StringBuilder();
				temp.append(en.getKey()).append("=");
				handleCharacter(value.getClass(),temp,value);
				if ( action.isWhereParameter(en.getKey()) ) {
					where.append(temp).append(" and ");
				} 
			}
		}
		int point = where.lastIndexOf(" and ");
		where.delete(point, point+6);
		
		if( "".equals(where) ) {
			return "";
		}
		
		StringBuilder sql = new StringBuilder();
		sql.append("delete from")
			.append(" ")
			.append(osm.getTableName())
			.append(" ")
			.append("where")
			.append(" ")
			.append(where);
		
		return sql.toString();
	}
	
	/**
	 * Assemble SQL for select action.
	 * @param fieldValueMap
	 * @param action
	 * @return
	 * @throws Exception
	 */
	private String makeSelectSQL(Map<String, Object> fieldValueMap,OSMObject osm, Action action) throws Exception {
		
		StringBuilder where = new StringBuilder(); //where sql
		StringBuilder temp = null;
		Object value = null;//value of string type
		
		Set<Entry<String,Object>> fieldValueSet = fieldValueMap.entrySet();
		for ( Entry<String,Object> en : fieldValueSet ) {
			value = en.getValue();
			if ( value != null ) {
				temp = new StringBuilder();
				//get value
				temp.append(en.getKey()).append("=");
				//whether or not use '' around character 
				DefaultBaseAction.handleCharacter(value.getClass(),temp,value);
				//join set sub sentence and where sub setnece 
				if ( action.isWhereParameter(en.getKey()) ) {
					where.append(temp).append(" and ");
				} 
			}
		}
		
		int point = where.lastIndexOf(" and ");
		where.delete(point, point+6);
		
		if( "".equals(where) ) {
			return "";
		}
		
		StringBuilder sql = new StringBuilder();
		sql.append("select * from")
			.append(" ")
			.append(osm.getTableName())
			.append(" ")
			.append("where")
			.append(" ")
			.append(where);
		
		return sql.toString();
	}

	/**
	 * Assemble SQL for prepared insert action.
	 * @param fieldValueMap
	 * @param osm
	 * @return
	 * @throws Exception
	 */
	private String makePreparedInsertSQL(Map<String, Object> fieldValueMap,OSMObject osm) throws Exception {
		
		StringBuilder dbField = new StringBuilder();
		StringBuilder values = new StringBuilder();
		Object value = null;
		
		@SuppressWarnings("unchecked")
		List<Object> valueList = (List<Object>)fieldValueMap.get(IOrmMaker.VALUES);
		@SuppressWarnings("unchecked")
		List<Class<?>> typeList = (List<Class<?>>)fieldValueMap.get(IOrmMaker.TYPES);
				
		Set<Entry<String,Object>> fieldValueSet = fieldValueMap.entrySet();
		for ( Entry<String,Object> en : fieldValueSet ) {
			if ( IOrmMaker.VALUES.equals(en.getKey()) || 
					IOrmMaker.TYPES.equals(en.getKey()) ) {
				continue;
			}
			value = en.getValue();
			if ( value != null ) {
				dbField.append(en.getKey()).append(",");
				values.append("?,");	
				valueList.add(value);
				typeList.add(value.getClass());
			}
		}
		int point = dbField.lastIndexOf(",");
		dbField.delete(point, point+1);
		point = values.lastIndexOf(",");
		values.delete(point, point+1);
		
		//sql
		StringBuilder sql = new StringBuilder();
		sql.append("insert into")
			.append(" ")
			.append(osm.getTableName())
			.append("(")
			.append(dbField.toString())
			.append(") values (")
			.append(values.toString())
			.append(")");
		
		return sql.toString();
	}
	
	/**
	 * Assemble SQL for prepared update action.
	 * @param fieldValueMap
	 * @param action
	 * @return
	 * @throws Exception
	 */
	private String makePreparedUpdateSQL(Map<String, Object> fieldValueMap,OSMObject osm, Action action) throws Exception {
		
		StringBuilder set = new StringBuilder();
		StringBuilder where = new StringBuilder(); 
		StringBuilder temp = null;
		Object value = null;
		
		@SuppressWarnings("unchecked")
		List<Object> valueList = (List<Object>)fieldValueMap.get(IOrmMaker.VALUES);
		@SuppressWarnings("unchecked")
		List<Class<?>> typeList = (List<Class<?>>)fieldValueMap.get(IOrmMaker.TYPES);
		List<Object> valueListWhere = new ArrayList<Object>();
		List<Class<?>> typeListWhere = new ArrayList<Class<?>>();
				
		Set<Entry<String,Object>> fieldValueSet = fieldValueMap.entrySet();
		for ( Entry<String,Object> en : fieldValueSet ) {
			if ( IOrmMaker.VALUES.equals(en.getKey()) || 
					IOrmMaker.TYPES.equals(en.getKey()) ) {
				continue;
			}
			value = en.getValue();
			if ( value != null ) {
				temp = new StringBuilder();
				temp.append(en.getKey()).append("=");
				//handleCharacter(value.getClass(),temp,value);
				temp.append("?");
				if ( action.isWhereParameter(en.getKey()) ) {
					where.append(temp).append(" and ");
					valueListWhere.add(value);
					typeListWhere.add(value.getClass());
				} else {
					set.append(temp).append(",");
					valueList.add(value);
					typeList.add(value.getClass());
				}
			}
		}
		int point = set.lastIndexOf(",");
		set.delete(point, point+1);
		point = where.lastIndexOf(" and ");
		where.delete(point, point+6);
		
		if( "".equals(where) ) {
			return "";
		}
		
		StringBuilder sql = new StringBuilder();
		sql.append("update")
			.append(" ")
			.append(osm.getTableName())
			.append(" ")
			.append("set")
			.append(" ")
			.append(set)
			.append(" ")
			.append("where")
			.append(" ")
			.append(where);
		
		valueList.addAll(valueListWhere);
		typeList.addAll(typeListWhere);
		
		return sql.toString();
	}
	
	/**
	 * Assemble SQL for prepared delete action.
	 * @param fieldValueMap
	 * @param action
	 * @return
	 * @throws Exception
	 */
	private String makePreparedDeleteSQL(Map<String, Object> fieldValueMap,OSMObject osm, Action action) throws Exception {
		
		Object value = null;
		StringBuilder where = new StringBuilder(); 
		StringBuilder temp = null;
		
		@SuppressWarnings("unchecked")
		List<Object> valueList = (List<Object>)fieldValueMap.get(IOrmMaker.VALUES);
		@SuppressWarnings("unchecked")
		List<Class<?>> typeList = (List<Class<?>>)fieldValueMap.get(IOrmMaker.TYPES);
		
		Set<Entry<String,Object>> fieldValueSet = fieldValueMap.entrySet();
		for ( Entry<String,Object> en : fieldValueSet ) {
			if ( IOrmMaker.VALUES.equals(en.getKey()) || 
					IOrmMaker.TYPES.equals(en.getKey()) ) {
				continue;
			}
			value = en.getValue();
			if ( value != null ) {
				if ( action.isWhereParameter(en.getKey()) ) {
					temp = new StringBuilder();
					temp.append(en.getKey()).append("=");
					//handleCharacter(value.getClass(),temp,value);
					temp.append("?");
					where.append(temp).append(" and ");
					valueList.add(value);
					typeList.add(value.getClass());
				} 
			}
		}
		int point = where.lastIndexOf(" and ");
		where.delete(point, point+6);
		
		if( "".equals(where) ) {
			return "";
		}
		
		StringBuilder sql = new StringBuilder();
		sql.append("delete from")
			.append(" ")
			.append(osm.getTableName())
			.append(" ")
			.append("where")
			.append(" ")
			.append(where);
		
		return sql.toString();
	}
	
	/**
	 * Assemble SQL for prepared select action.
	 * @param fieldValueMap
	 * @param action
	 * @return
	 * @throws Exception
	 */
	private String makePreparedSelectSQL(Map<String, Object> fieldValueMap,OSMObject osm, Action action) throws Exception {
		
		StringBuilder where = new StringBuilder(); //where sql
		StringBuilder temp = null;
		Object value = null;//value of string type
		
		@SuppressWarnings("unchecked")
		List<Object> valueList = (List<Object>)fieldValueMap.get(IOrmMaker.VALUES);
		@SuppressWarnings("unchecked")
		List<Class<?>> typeList = (List<Class<?>>)fieldValueMap.get(IOrmMaker.TYPES);
		
		Set<Entry<String,Object>> fieldValueSet = fieldValueMap.entrySet();
		for ( Entry<String,Object> en : fieldValueSet ) {
			if ( IOrmMaker.VALUES.equals(en.getKey()) || 
					IOrmMaker.TYPES.equals(en.getKey()) ) {
				continue;
			}
			value = en.getValue();
			if ( value != null ) {
				if ( action.isWhereParameter(en.getKey()) ) {
					temp = new StringBuilder();
					//get value
					temp.append(en.getKey()).append("=");
					//whether or not use '' around character 
					//DefaultBaseAction.handleCharacter(value.getClass(),temp,value);
					temp.append("?");
					//join set sub sentence and where sub setnece 
					where.append(temp).append(" and ");
					valueList.add(value);
					typeList.add(value.getClass());
				} 
			}
		}
		
		int point = where.lastIndexOf(" and ");
		where.delete(point, point+6);
		
		if( "".equals(where) ) {
			return "";
		}
		
		StringBuilder sql = new StringBuilder();
		sql.append("select * from")
			.append(" ")
			.append(osm.getTableName())
			.append(" ")
			.append("where")
			.append(" ")
			.append(where);
		
		return sql.toString();
	}

	@Override
	public List<Object> doQueryAction(String actionName, Object obj,
			Connection conn, boolean isPrepared) throws ExecuteSQLException {
		Action act = ConfigLoader.getInstance().getAction(actionName);
		try {
			String sql = act.orm2PreparedSQL(this, obj);
			return this.query(sql, obj.getClass(), conn);
		} catch (Exception e) {
			logger.error("[DefaultOrmPlusAction:doQueryAction]That transformation orm to sql is error.", e);
		}
		return null;
	}
}
