/**
 * Persistence2 is an easy-use persistence framework. There are more interfaces 
 * and classes of ORM and SQL-MAPPING than old version and it provides more solutions 
 * for quick-development of complex application. 
 * 
 * Persistence2-obj2sql is the core toolkit of Persistence2 framework. It provides 
 * more services to deal with complex business logic by ORM and SQL-MAPPING. It 
 * would help developers to release from coding of DAO.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.persistence2.sql.tools;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.connection.exception.OperationsException;
import com.nonesole.persistence2.connection.pool.MultiConnectionPool;
import com.nonesole.persistence2.sql.PersistenceFactory;
import com.nonesole.persistence2.sql.define.IOrmAction;
import com.nonesole.persistence2.sql.define.IOrmPlusAction;
import com.nonesole.persistence2.sql.define.IVoidAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;

/**
 * This class is used to deal with database transaction.
 * @author jack lee
 * @version 1.0
 */
public class PSession {

	private Connection conn;
	private IOrmAction oAction;
	private IOrmPlusAction opAction;
	private IVoidAction voidAction;
	
	/**
	 * Constructor
	 */
	public PSession(){
		try {
			conn = PersistenceFactory.getInstance().getDefaultConnection();
			if ( conn == null ) {
				conn = MultiConnectionPool.getInstance().getConnection(
						MultiConnectionPool.getInstance().getDefaultDataSourceName());
			}
			if ( conn == null ) {
				throw new OperationsException("PSession can not get default connection.");
			}
			conn.setAutoCommit(false);
			oAction = PersistenceFactory.getInstance().getOrmAction();
			opAction = PersistenceFactory.getInstance().getOrmPlusAction();
			voidAction = PersistenceFactory.getInstance().getVoidAction();
		} catch (Exception e) {
			Logger.getInstance().error("PSession has an error.", e);
			
		}
	}
	
	/**
	 * commit
	 */
	public void commit(){
		try {
			conn.commit();
		} catch (Exception e) {
			Logger.getInstance().error("PSession has an error when connection is commited.", e);
			if ( conn != null ) {
				try {
					conn.rollback();
				} catch (SQLException e1) {
					Logger.getInstance().error("PSession has an error when connection rollback.", e1);
					e1.printStackTrace();
				}
			}
		} finally {
			if ( conn != null ) {
				try {
					conn.setAutoCommit(true);
					conn.close();
				} catch (SQLException e2) {
					Logger.getInstance().error("PSession has an error when connection close.", e2);
					e2.printStackTrace();
				}
				
			}
		}
	}
	
	/**
	 * This method is used to solve persistence of list, map and POJO, 
	 * it depends on IVoidAction interface.
	 * @param action
	 * @param actionName
	 * @param object
	 * @return PSession
	 */
	public PSession voidAction(String actionName, Object object) {
		try {
			voidAction.voidAction(actionName, object, conn);
		} catch (ExecuteSQLException e) {
			Logger.getInstance().error("PSession has an error when sys executes sql.", e);
		}
		return this;
	}
	
	/**
	 * This method is used to insert object, it depends on IOrmAction interface.
	 * @param object
	 * @return PSession
	 */
	public PSession insert(Object object) {
		try {
			oAction.insert(object, conn);
		} catch (ExecuteSQLException e) {
			Logger.getInstance().error("PSession has an error when sys inserts object.", e);
		}
		return this;
	}
	
	/**
	 * This method is used to update object, it depends on IOrmAction interface.
	 * @param object
	 * @return PSession
	 */
	public PSession update(Object object) {
		try {
			oAction.update(object, conn);
		} catch (ExecuteSQLException e) {
			Logger.getInstance().error("PSession has an error when sys updates object.", e);
		}
		return this;
	}
	
	/**
	 * @param object
	 * @return PSession
	 */
	public PSession delete(Object object) {
		try {
			oAction.delete(object, conn);
		} catch (ExecuteSQLException e) {
			Logger.getInstance().error("PSession has an error when sys updates object.", e);
		}
		return this;
	}
	
	/**
	 * This method is used to execute DML, it depends on IOrmPlusAction interface.
	 * @param object
	 * @return PSession
	 */
	public PSession doAction(String actionName, Object object) {
		try {
			opAction.doAction(actionName, object, conn);
		} catch (ExecuteSQLException e) {
			Logger.getInstance().error("PSession has an error when sys does action.", e);
			
		}
		return this;
	}
	
	/**
	 * This method is used to do batch execution of DML, it depends on IOrmPlusAction interface.
	 * @param object
	 * @return PSession
	 */
	public PSession doBatchAction(String actionName, List<Object> objects) {
		try {
			opAction.doBatchAction(actionName, objects, conn);
		} catch (ExecuteSQLException e) {
			Logger.getInstance().error("PSession has an error when sys does batch action.", e);
			
		}
		return this;
	}
	
}
